#! /usr/bin/python
# -*- coding: utf-8 -*-



# import funkcí z jiného adresáře
import argparse
import os.path

path_to_script = os.path.dirname(os.path.abspath(__file__))
# sys.path.append(os.path.join(path_to_script, "../extern/pyseg_base/src"))
# sys.path.append(os.path.join(path_to_script, "../extern/sed3/"))
#sys.path.append(os.path.join(path_to_script, "../extern/"))
#import featurevector

import logging
logger = logging.getLogger(__name__)


#import apdb
#  apdb.set_trace();\
#import scipy.io
import numpy as np
import scipy
import scipy.ndimage
from imtools import misc, qmisc


# ----------------- my scripts --------
class BodyNavigation:

    def __init__(self, data3d, voxelsize_mm):
        self.voxelsize_mm = np.asarray(voxelsize_mm)
        self.working_vs = np.asarray([1.5, 1.5, 1.5])
        if voxelsize_mm is None:
            self.data3dr = data3d
        else:
            self.data3dr = qmisc.resize_to_mm(data3d, voxelsize_mm, self.working_vs)
        self.lungs = None
        self.spine = None
        self.body = None
        self.orig_shape = data3d.shape
        self.diaphragm_mask = None
        self.angle = None


    def get_spine(self):

        spine = scipy.ndimage.filters.gaussian_filter(self.data3dr, sigma=[20, 5, 5]) > 200
        self.spine = spine

        self.spine_center = np.mean(np.nonzero(self.spine), 1)
        # self.center2 = np.mean(np.nonzero(self.spine), 2)
        return qmisc.resize_to_shape(spine, self.orig_shape)

    def get_body(self):
        body = scipy.ndimage.filters.gaussian_filter(self.data3dr, sigma=2) > -150
        body[0, :, :] = 1
        body[-1, :, :] = 1

        self.body = scipy.ndimage.morphology.binary_fill_holes(body)
        return qmisc.resize_to_shape(self.body, self.orig_shape)

    def get_lungs(self):
        lungs = scipy.ndimage.filters.gaussian_filter(self.data3dr, sigma=[4, 2, 2]) > -150
        lungs[0, :, :] = 1

        lungs = scipy.ndimage.morphology.binary_fill_holes(lungs)
        labs, n = scipy.ndimage.measurements.label(lungs==0)
        cornerlab = [
            labs[0,0,0],
            labs[0,0,-1],
            labs[0,-1,0],
            labs[0,-1,-1],
            labs[-1,0,0],
            labs[-1,0,-1],
            labs[-1,-1,0],
            labs[-1,-1,-1]
            ]

        lb = np.median(cornerlab)
        labs[labs==lb] = 0

        labs[labs==labs[0,0,0]] = 0
        labs[labs==labs[0,0,-1]] = 0
        labs[labs==labs[0,-1,0]] = 0
        labs[labs==labs[0,-1,-1]] = 0
        labs[labs==labs[-1,0,0]] = 0
        labs[labs==labs[-1,0,-1]] = 0
        labs[labs==labs[-1,-1,0]] = 0
        labs[labs==labs[-1,-1,-1]] = 0

        lungs = labs > 0
        self.lungs = lungs
        #self.body = (labs == 80)
        return misc.resize_to_shape(lungs, self.orig_shape)

    def dist_to_surface(self):
        if self.body is None:
            self.get_body()
        ld = scipy.ndimage.morphology.distance_transform_edt(self.body)

        return misc.resize_to_shape(ld, self.orig_shape)

    def dist_to_lungs(self):
        if self.lungs is None:
            self.get_lungs()

        ld = scipy.ndimage.morphology.distance_transform_edt(1 - self.lungs)
        return misc.resize_to_shape(ld, self.orig_shape)

    def dist_to_spine(self):
        if self.spine is None:
            self.get_spine()
        ld = scipy.ndimage.morphology.distance_transform_edt(1 - self.spine)
        return misc.resize_to_shape(ld, self.orig_shape)

    def find_symmetry(self, degrad=5, return_img=False):
        img = np.sum(self.data3dr > 430, axis=0)
        tr0, tr1, angle = find_symmetry(img, degrad)
        self.angle = angle
        self.symmetry_point = np.array([tr0, tr1])
        if return_img:
            return img


    def dist_sagittal(self, degrad=5):
        if self.angle is None:
            self.find_symmetry()
        rldst = np.ones(self.orig_shape, dtype=np.int16)
        symmetry_point_orig_res = self.symmetry_point * self.working_vs[1:] / self.voxelsize_mm[1:].astype(np.double)

        z = split_with_line(symmetry_point_orig_res, self.angle , self.orig_shape[1:])
        # print 'z  ', np.max(z), np.min(z)

        for i in range(self.orig_shape[0]):
            rldst[i ,: ,:] = z

        # rldst = scipy.ndimage.morphology.distance_transform_edt(rldst) - int(spine_mean[2])
        # return misc.resize_to_shape(rldst, self.orig_shape)
        return rldst

    def dist_coronal(self):
        if self.spine is None:
            self.get_spine()
        if self.angle is None:
            self.find_symmetry()
        spine_mean = np.mean(np.nonzero(self.spine), 1)
        rldst = np.ones(self.orig_shape, dtype=np.int16)
        # rldst = np.ones(self.data3dr.shape, dtype=np.int16)
        # rldst[:, 0, :] = 0

        spine_center = spine_mean * self.working_vs / self.voxelsize_mm.astype(np.double)

        # rldst = scipy.ndimage.morphology.distance_transform_edt(rldst) - int(spine_mean[1])

        z = split_with_line(spine_center[1:], self.angle + 90 , self.orig_shape[1:])
        for i in range(self.orig_shape[0]):
            rldst[i ,: ,:] = z

        return rldst

    def dist_axial(self):
        if self.diaphragm_mask is None:
            self.get_diaphragm_mask()
        axdst = np.ones(self.data3dr.shape, dtype=np.int16)
        axdst[0 ,: ,:] = 0
        iz, ix, iy = np.nonzero(self.diaphragm_mask)
        # print 'dia level ', self.diaphragm_mask_level

        axdst = scipy.ndimage.morphology.distance_transform_edt(axdst) - int(self.diaphragm_mask_level)
        return misc.resize_to_shape(axdst, self.orig_shape)



    def dist_diaphragm(self):
        if self.diaphragm_mask is None:
            self.get_diaphragm_mask()
        dst = (scipy.ndimage.morphology.distance_transform_edt(
                self.diaphragm_mask)
               -
               scipy.ndimage.morphology.distance_transform_edt(
                1 - self.diaphragm_mask)
              )
        return qmisc.resize_to_shape(dst, self.orig_shape)

    def _get_ia_ib_ic(self, axis):
        """
        according to axis gives order of of three dimensions
        :param axis: 0, 1 or 2 is allowed
        :return:
        """
        if axis == 0:
            ia = 0
            ib = 1
            ic = 2
        elif axis == 1:
            ia = 1
            ib = 0
            ic = 2
        elif axis == 2:
            ia = 2
            ib = 0
            ic = 1
        else:
            logger.error('Unrecognized axis')


        return ia, ib, ic

    def _filter_diaphragm_profile_image_remove_outlayers(self, profile, axis=0, tolerance=80):
        # import bottleneck
        # tolerance * 1.5mm

        med = np.median(profile[profile > 0])
        profile[np.abs(profile - med) > tolerance] = 0
        return profile

    def get_diaphragm_profile_image_with_empty_areas(self, axis=0):
        if self.lungs is None:
            self.get_lungs()
        if self.spine is None:
            self.get_spine()
        if self.angle is None:
            self.find_symmetry()
        axis = 0
        data = self.lungs
        ia, ib, ic = self._get_ia_ib_ic(axis)

        # gradient
        gr = scipy.ndimage.filters.sobel(data.astype(np.int16), axis=ia)
        # seg = (np.abs(ss.dist_coronal()) > 20).astype(np.uint8) + (np.abs(ss.dist_sagittal()) > 20).astype(np.uint8)
        grt = gr > 12
        # seg = (np.abs(self.dist_sagittal()) > zero_stripe_width).astype(np.uint8)
        # grt = grt * seg
        # nalezneme nenulove body
        nz = np.nonzero(grt)

        # udelame z 3d matice placku, kde jsou nuly tam, kde je nic a jinde jsou
        # z-tove souradnice
        flat = np.zeros([grt.shape[ib], grt.shape[ic]])
        flat[(nz[ib], nz[ic])] = [nz[ia]]


        # symmetry_point_orig_res = self.symmetry_point * self.working_vs[1:] / self.voxelsize_mm[1:].astype(np.double)
        # odstranime z dat pruh kolem srdce. Tam byva obcas jicen, nebo je tam oblast nad srdcem
        # symmetry_point_pixels = self.symmetry_point/ self.working_vs[1:]
        flat[flat==0] = np.NaN
        return flat

    def remove_pizza(self, flat, zero_stripe_width=10, alpha0=-20, alpha1=40 ):
        """
        Remove circular sector from the image with center in spine
        :param flat: input 2D image
        :param zero_stripe_width: offset to pivot point. Pizza line is zero_stripe_width far
        :param alpha0: Additional start angle relative to detected orientation
        :param alpha1: Additional end angle relative to detected orientation
        :return:
        """
        spine_mean = np.mean(np.nonzero(self.spine), 1)
        spine_mean = spine_mean[1:]

        z1 = split_with_line(spine_mean, self.angle + alpha1, flat.shape)
        z2 = split_with_line(spine_mean, self.angle + alpha0, flat.shape)

        z1 = (z1 > zero_stripe_width).astype(np.int8)
        z2 = (z2 < -zero_stripe_width).astype(np.int8)
        # seg = (np.abs(z) > zero_stripe_width).astype(np.int)
        seg = (z1 * z2)

        flat [seg>0]= np.NaN # + seg*10
        # print 'sp ', spine_mean
        # print 'sporig ', symmetry_point_orig_res
        # flat = seg

        return flat

    def filter_remove_outlayers(self, flat, minimum_value=0):
        """
        Remove outlayers using ellicptic envelope from scikits learn
        :param flat:
        :param minimum_value:
        :return:
        """
        from sklearn.covariance import EllipticEnvelope
        flat0 = flat.copy()
        flat0[np.isnan(flat)] = 0
        x,y = np.nonzero(flat0)
        # print np.prod(flat.shape)
        # print len(y)

        z = flat[(x,y)]

        data = np.asarray([x,y,z]).T

        clf = EllipticEnvelope(contamination=.1)
        clf.fit(data)
        y_pred = clf.decision_function(data)


        out_inds = y_pred < minimum_value
        flat[(x[out_inds], y[out_inds])] = np.NaN
        return flat


    def filter_ignoring_nan(self, flat, kernel_size_mm=None , max_dist_mm=30):
        """
        Compute filtered plane and removes pixels wiht distance grater then max_dist_mm

        :param flat:
        :param kernel_size_mm:
        :param max_dist_mm:
        :return:
        """

        if kernel_size_mm is None: kernel_size = [150, 150]

        # kernel_size must be odd - lichý
        kernel_size = np.asarray(kernel_size_mm) / self.working_vs[1:]
        # print 'ks1 ', kernel_size
        odd = kernel_size % 2
        kernel_size = kernel_size + 1 - odd
        # print 'ks2 ', kernel_size

        # metoda 1
        kernel = np.ones(kernel_size)
        kernel = kernel / (1.0 * np.prod(kernel_size))
        # flat = scipy.ndimage.filters.convolve(flat, kernel)


        # metoda 2
        # # flat = flat.reshape([flat.shape[0], flat.shape[1], 1])
        import astropy.convolution
        flat_out = astropy.convolution.convolve(flat, kernel, boundary='extend')

        too_bad_pixels = np.abs(flat_out - flat) > (max_dist_mm/self.working_vs[0])


        flat[too_bad_pixels] = np.NaN
        # metoda 3
        # doplnime na nenulova mista střední hodnotu
        # flat_mask = np.isnan(flat)
        #
        # mn = np.mean(flat[flat_mask == False])
        #
        # flat_copy = flat.copy()
        # flat_copy[flat_mask] = mn
        #
        # flat_copy = scipy.ndimage.filters.gaussian_filter(flat_copy, sigma=sigma)
        # flat = flat_copy






        return flat


    def get_diaphragm_profile_image(self, axis=0, preprocessing=True):
        flat = self.get_diaphragm_profile_image_with_empty_areas(axis)

        if preprocessing:
            flat = self.remove_pizza(flat)
            flat = self._filter_diaphragm_profile_image_remove_outlayers(flat)
            flat = self.filter_ignoring_nan(flat)
            flat = self.filter_remove_outlayers(flat)



        # jeste filtrujeme ne jen podle stredni hodnoty vysky, ale i v prostoru
        # flat0 = flat==0
        # flat[flat0] = None
        #
        # flat = scipy.ndimage.filters.median_filter(flat, size=(40,40))
        # flat[flat0] = 0


        # something like interpolation
        # doplnime praznda mista v ploche mape podle nejblizsi oblasi
        # filter with big mask
        ou = fill_nan_with_nearest(flat.copy())
        ou = scipy.ndimage.filters.gaussian_filter(ou, sigma=5)


        # get back valid values on its places
        valid_flat_inds = 1 - np.isnan(flat)
        ou[valid_flat_inds] = flat[valid_flat_inds]
        # flat = ou

        ou = fill_nan_with_nearest(ou)
        # overal filter
        ou = scipy.ndimage.filters.median_filter(ou, size=(5,5))
        ou = scipy.ndimage.filters.gaussian_filter(ou, sigma=3)

        # ou = self.__filter_diaphragm_profile_image(ou, axis)
        return ou


    def __filter_diaphragm_profile_image(self, profile, axis=0):
        """
        filter do not go down in compare to pixel near to the back
        :param profile:
        :param axis:
        :return:
        """
        if axis == 0:

            profile_w = profile.copy()

            # profile_out = np.zeros(profile.shape)
            for i in range(profile_w.shape[0] -1 , 0 , -1):
                profile_line_0 = profile_w[i, :]
                profile_line_1 = profile_w[i - 1, :]
                where_is_bigger = profile_line_1  < (profile_line_0 - 0)
            #     profile_line_out[where_is_bigger] = profile_line_0[where_is_bigger]
                profile_w[i - 1, where_is_bigger] = profile_line_0[where_is_bigger]
                profile_w[i - 1, np.negative(where_is_bigger)] = profile_line_1[np.negative(where_is_bigger)]
            #     profile_out[where_is_bigger, :] = profile_line_1
        else:
            logger.error('other axis not implemented yet')

        return profile_w
        # plt.imshow(profile_w, cmap='jet')

    def get_diaphragm_mask(self, axis=0):
        if self.lungs is None:
            self.get_lungs()
        ia, ib, ic = self._get_ia_ib_ic(axis)
        data = self.lungs
        ou = self.get_diaphragm_profile_image(axis=axis)
        # reconstruction mask array
        mask = np.zeros(data.shape)
        for i in range(mask.shape[ia]):
            if ia == 0:
                mask[i,:,:] = ou > i
            elif ia == 1:
                mask[:,i,:] = ou > i
            elif ia == 2:
                mask[:,:,i] = ou > i

        self.diaphragm_mask = mask

        # maximal point is used for axial ze
        # ro plane
        self.diaphragm_mask_level = np.median(ou)
        self.center0 = self.diaphragm_mask_level * self.working_vs[0]

        return misc.resize_to_shape(self.diaphragm_mask, self.orig_shape)

    def get_center(self):
        self.get_diaphragm_mask()
        self.get_spine()

        self.center = np.array([self.diaphragm_mask_level, self.spine_center[0], self.spine_center[1]])
        self.center_mm = self.center * self.working_vs
        self.center_orig = self.center * self.voxelsize_mm / self.working_vs.astype(np.double)

        return self.center_orig


def prepare_images(imin0, pivot):
    imin1 = imin0[:,::-1]
    img=imin0

    padX0 = [img.shape[1] - pivot[1], pivot[1]]
    padY0 = [img.shape[0] - pivot[0], pivot[0]]
    imgP0 = np.pad(img, [padY0, padX0], 'constant')


    img = imin1
    padX1 = [pivot[1], img.shape[1] - pivot[1]]
    padY1 = [img.shape[0] - pivot[0], pivot[0]]
    imgP1 = np.pad(img, [padY1, padX1], 'constant')
    return imgP0, imgP1

def find_symmetry_parameters(imin0, trax, tray, angles):
    vals = np.zeros([len(trax), len(tray), len(angles)])
#     angles_vals = []



    for i, x in enumerate(trax):
#         print 'x ', x
        for j, y in enumerate(tray):
            try:
                img = imin0
                pivot=[x,y]
                imgP0, imgP1 = prepare_images(imin0, pivot)

    #             print 'y ', y
                for k, angle in enumerate(angles):

    #             print angle
#                 print imin0.shape, imin.shape, angle, x, y
#                     imr=rotateImage(imin.astype(int), angle, [imin0.shape[0] - x, y])
#                     dif = (imr-imin0)**2
#                     sm = np.sum(dif)
                    imr = scipy.ndimage.rotate(imgP1, angle, reshape=False)
                    dif = (imgP0-imr)**2
                    sm = np.sum(dif)
                    vals[i,j,k] = sm
            except:
                vals[i,j,:] = np.inf
    #             angles_vals.append(sm)

    # am = np.argmin(angles_vals)
    am = np.unravel_index(np.argmin(vals), vals.shape)
    # print am, ' min ', np.min(vals)

    return trax[am[0]], tray[am[1]], angles[am[2]]

def find_symmetry(img, degrad=5):
    imin0r = scipy.misc.imresize(img, np.asarray(img.shape)/degrad)

    angles = range(-180,180,15)
    trax = range(1, imin0r.shape[0],10)
    tray = range(1, imin0r.shape[1],10)



    tr0, tr1, ang = find_symmetry_parameters(imin0r, trax, tray, angles)


    # fine measurement
    angles = range(ang-20,ang+20, 3)
    trax = range(np.max([tr0-20, 1]), tr0+20, 3)
    tray = range(np.max([tr1-20, 1]), tr1+20, 3)

    tr0, tr1, ang = find_symmetry_parameters(imin0r, trax, tray, angles)

    angle = 90-ang/2.0

    return tr0*degrad, tr1*degrad, angle


# Rozděl obraz na půl
def split_with_line(point, orientation, imshape, degrees=True):
    """
    :arg point:
    :arg orientation: angle or oriented vector
    :arg degrees: if is set to True inptu angle is expected to be in radians, default True
    """

    if np.isscalar(orientation):
        angle = orientation
        if degrees:
            angle = np.radians(angle)

        # kvadranty
        angle = angle % (2* np.pi)
               # kvadranty
        angle = angle % (2* np.pi)
        # print np.degrees(angle)
        if (angle > (0.5*np.pi)) and (angle < (1.5*np.pi)):

            zn = -1
        else:
            zn = 1

        vector = [np.tan(angle), 1]
        # vector = [np.tan(angle), 1]
    else:
        vector = orientation

    vector = vector / np.linalg.norm(vector)
    x, y = np.mgrid[:imshape[0], :imshape[1]]
#     k = -vector[1]/vector[0]
#     z = ((k * (x - point[0])) + point[1] - y)
    a = vector[1]
    b = -vector[0]

    c = -a * point[0] - b * point[1]


    z = zn * (a * x + b * y + c) / (a**2 + b**2)**0.5
    return z

def fill_nan_with_nearest(flat):
    indices = scipy.ndimage.morphology.distance_transform_edt(np.isnan(flat), return_indices=True, return_distances=False)
    # indices = scipy.ndimage.morphology.distance_transform_edt(flat==0, return_indices=True, return_distances=False)
    ou = flat[(indices[0],indices[1])]
    return ou

def main():

    #logger = logging.getLogger(__name__)
    logger = logging.getLogger()

    logger.setLevel(logging.WARNING)
    ch = logging.StreamHandler()
    logger.addHandler(ch)

    #logger.debug('input params')

    # input parser
    parser = argparse.ArgumentParser(description=
            'Segmentation of bones, lungs and heart.')
    parser.add_argument('-i','--datadir',
            default=None,
            help='path to data dir')
    parser.add_argument('-o','--output',
            default=None,
            help='output file')

    parser.add_argument('-d', '--debug', action='store_true',
            help='run in debug mode')
    parser.add_argument('-ss', '--segspine', action='store_true',
            help='run spine segmentaiton')
    parser.add_argument('-sb', '--segbody', action='store_true',
                        help='run body segmentaiton')
    parser.add_argument('-exd', '--exampledata', action='store_true',
            help='run unittest')
    parser.add_argument('-so', '--show_output', action='store_true',
            help='Show output data in viewer')
    args = parser.parse_args()



    if args.debug:
        logger.setLevel(logging.DEBUG)


    if args.exampledata:

        args.dcmdir = '../sample_data/liver-orig001.raw'

#    if dcmdir == None:

    #else:
    #dcm_read_from_dir('/home/mjirik/data/medical/data_orig/46328096/')
    #data3d, metadata = dcmr.dcm_read_from_dir(args.dcmdir)
    import io3d

    data3d , metadata = io3d.datareader.read(args.datadir)

    bn = BodyNavigation(data3d = data3d,
            voxelsize_mm = metadata['voxelsize_mm'],
            )


    seg = np.zeros(data3d.shape, dtype=np.int8)
    #sseg.orientation()
    if args.segspine:
        seg += bn.get_spine()
    if args.segspine:
        seg += bn.get_body()


    #print ("Data size: " + str(data3d.nbytes) + ', shape: ' + str(data3d.shape) )

    #igc = pycut.ImageGraphCut(data3d, zoom = 0.5)
    #igc.interactivity()

    #igc.make_gc()
    #igc.show_segmentation()

    # volume
    #volume_mm3 = np.sum(oseg.segmentation > 0) * np.prod(oseg.voxelsize_mm)


    #pyed = sed3.sed3(oseg.data3d, contour = oseg.segmentation)
    #pyed.show()

    if args.show_output:
        import sed3
        sed3.show_slices(data3d, contour=seg)


    #savestring = raw_input ('Save output data? (y/n): ')
    #sn = int(snstring)
    if args.output is not None: # savestring in ['Y','y']:
        md = {
            'voxelsize_mm': metadata['voxelsize_mm'],
            'segmentation': seg, }

        io3d.write(data3d, args.output, metadata=md)

    #output = segmentation.vesselSegmentation(oseg.data3d, oseg.orig_segmentation)


if __name__ == "__main__":
    main()



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright © %YEAR% %USER% <%MAIL%>
#
# Distributed under terms of the %LICENSE% license.

"""
%HERE%
"""

import logging

logger = logging.getLogger(__name__)
import argparse

import sys
from PyQt4.QtCore import *
from PyQt4.QtGui import *


class LogEntryModel(QAbstractListModel):
    def __init__(self, logfile, parent=None):
        super(LogEntryModel, self).__init__(parent)
        print logfile
        self.parent = parent
        self.entries = None
        self.slurp(logfile)
        self.logfile = logfile

    def rowCount(self, parent=QModelIndex()):
        return len(self.entries)

    def data(self, index, role):
        # print "data is called ", index.row()
        if index.isValid() and role == Qt.DisplayRole:
            return QVariant(self.entries[index.row()])
        else:
            return QVariant()

    def slurp(self, logfile=None):

        if self.entries is None:
            self.entries = []
        if logfile is None:
            logfile=self.logfile
        with open(logfile, 'rb') as fp:
            for line in fp.readlines():
                # tokens = line.strip().split(' : ')
                # sender = tokens[2]
                # message = tokens[4]
                # entry = "%s %s" % (sender, message)
                entry = line
                self.entries.append(entry)
            # print str(self.entries[-1]), len(self.entries)
            # self.parent.addItem("adf")
            # self.parent.update()
            # self.appendRow(self.enteries[-1])



class LogViewerForm(QDialog):
    def __init__(self, logfile, qapp=None, parent=None):
        super(LogViewerForm, self).__init__(parent)

        self.watcher = QFileSystemWatcher([logfile], parent=None)
        self.connect(self.watcher, SIGNAL('fileChanged(const QString&)'), self.update_log)
        self.qapp = qapp

        # build the list widget
        list_label = QLabel(QString("<strong>MoMo</strong> Log Viewer"))
        list_model = LogEntryModel(logfile)
        self.list_model = list_model
        self.list_view = QListView()
        self.list_view.setModel(self.list_model)
        list_label.setBuddy(self.list_view)

        # define the layout
        layout = QVBoxLayout()
        layout.addWidget(list_label)
        layout.addWidget(self.list_view)
        self.setLayout(layout)

    def update_log(self):
        print 'file changed'
        self.list_model.slurp(self.list_model.logfile)
        self.list_view.updateGeometries()
        self.list_view.update()
        self.list_view.repaint()

        self.qapp.processEvents()
        # from PyQt4.QtCore import pyqtRemoveInputHook; pyqtRemoveInputHook()
        # import ipdb; ipdb.set_trace()
        from PyQt4 import QtCore
        # QTCore

def main():
    logger = logging.getLogger()

    logger.setLevel(logging.DEBUG)
    ch = logging.StreamHandler()
    logger.addHandler(ch)

    # create file handler which logs even debug messages
    # fh = logging.FileHandler('log.txt')
    # fh.setLevel(logging.DEBUG)
    # formatter = logging.Formatter(
    #     '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    # fh.setFormatter(formatter)
    # logger.addHandler(fh)
    # logger.debug('start')

    # input parser
    parser = argparse.ArgumentParser(
        description=__doc__
    )
    parser.add_argument(
        '-i', '--inputfile',
        default=None,
        required=True,
        help='input file'
    )
    parser.add_argument(
        '-d', '--debug', action='store_true',
        help='Debug mode')
    args = parser.parse_args()

    if args.debug:
        ch.setLevel(logging.DEBUG)

    app = QApplication(sys.argv)
    # form = LogViewerForm(args.inputfile, qapp=app)
    # form.show()




    list = QListView()
    list.setWindowTitle('nazev okna')
    list.setMinimumSize(600,400)
    model = LogEntryModel(args.inputfile, parent=list)
    list.setModel(model)
    watcher = QFileSystemWatcher([args.inputfile], parent=None)

    list.connect(watcher, SIGNAL('fileChanged(const QString&)'), model.slurp)

    list.show()
    app.exec_()

if __name__ == "__main__":
    main()




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)

import vtk
import numpy as nm
import yaml
import sys
import argparse

#
# def get_cylinder(upper, height, radius,
#                  direction,
#                  resolution=10):
#
#     src = vtk.vtkCylinderSource()
#     src.SetCenter((0, height/2, 0))
#     src.SetHeight(height + radius/2.0)
#     src.SetRadius(radius)
#     src.SetResolution(resolution)
#
#     rot1 = vtk.vtkTransform()
#     fi = nm.arccos(direction[1])
#
#     rot1.RotateWXYZ(-nm.rad2deg(fi), 0.0, 0.0, 1.0)
#     u = nm.abs(nm.sin(fi))
#     rot2 = vtk.vtkTransform()
#     if u > 1.0e-6:
#
#         # sometimes d[0]/u little bit is over 1
#         d0_over_u = direction[0] / u
#         if d0_over_u > 1:
#             psi = 0
#         elif d0_over_u < -1:
#             psi = 2 * nm.pi
#         else:
#             psi = nm.arccos(direction[0] / u)
#
#         logger.debug('d0 '+str(direction[0])+'  u '+str(u)+' psi '+str(psi))
#         if direction[2] < 0:
#             psi = 2 * nm.pi - psi
#
#         rot2.RotateWXYZ(-nm.rad2deg(psi), 0.0, 1.0, 0.0)
#
#     tl = vtk.vtkTransform()
#     tl.Translate(upper)
#
#     tr1a = vtk.vtkTransformFilter()
#     tr1a.SetInput(src.GetOutput())
#     tr1a.SetTransform(rot1)
#
#     tr1b = vtk.vtkTransformFilter()
#     tr1b.SetInput(tr1a.GetOutput())
#     tr1b.SetTransform(rot2)
#
#     tr2 = vtk.vtkTransformFilter()
#     tr2.SetInput(tr1b.GetOutput())
#     tr2.SetTransform(tl)
#
#     tr2.Update()
#
#     return tr2.GetOutput()
#
#
# def gen_tree(tree_data):
#
#     points = vtk.vtkPoints()
#     polyData = vtk.vtkPolyData()
#     polyData.Allocate(1000, 1)
#     polyData.SetPoints(points)
#     poffset = 0
#
#     for br in tree_data:
#         cyl = get_cylinder(br['upperVertex'],
#                            br['length'],
#                            br['radius'],
#                            br['direction'],
#                            resolution=16)
#
#         for ii in xrange(cyl.GetNumberOfPoints()):
#             points.InsertPoint(poffset + ii, cyl.GetPoint(ii))
#
#         for ii in xrange(cyl.GetNumberOfCells()):
#             cell = cyl.GetCell(ii)
#             cellIds = cell.GetPointIds()
#             for jj in xrange(cellIds.GetNumberOfIds()):
#                 oldId = cellIds.GetId(jj)
#                 cellIds.SetId(jj, oldId + poffset)
#
#             polyData.InsertNextCell(cell.GetCellType(),
#                                     cell.GetPointIds())
#
#         poffset += cyl.GetNumberOfPoints()
#
#     return polyData
#
#
# def process_tree(indata):
#     scale = 1e-3
#     scale = 1
#
#     outdata = []
#     for key in indata:
#         ii = indata[key]
#         logger.debug(ii)
#         br = {}
#         try:
#             # old version of yaml tree
#             vA = ii['upperVertexXYZmm']
#             vB = ii['lowerVertexXYZmm']
#             radi = ii['radius']
#             lengthEstimation = ii['length']
#         except:
#             # new version of yaml
#             try:
#                 vA = ii['nodeA_ZYX_mm']
#                 vB = ii['nodeB_ZYX_mm']
#                 radi = ii['radius_mm']
#                 lengthEstimation = ii['lengthEstimation']
#             except:
#                 continue
#
#         br['upperVertex'] = nm.array(vA) * scale
#         br['radius'] = radi * scale
#         br['real_length'] = lengthEstimation * scale
#
#         vv = nm.array(vB) * scale - br['upperVertex']
#         br['direction'] = vv / nm.linalg.norm(vv)
#         br['length'] = nm.linalg.norm(vv)
#         outdata.append(br)
#
#     return outdata

def main():
    logger = logging.getLogger()

    logger.setLevel(logging.DEBUG)
    ch = logging.StreamHandler()
    logger.addHandler(ch)

    # create file handler which logs even debug messages
    # fh = logging.FileHandler('log.txt')
    # fh.setLevel(logging.DEBUG)
    # formatter = logging.Formatter(
    #     '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    # fh.setFormatter(formatter)
    # logger.addHandler(fh)
    # logger.debug('start')

    # input parser
    parser = argparse.ArgumentParser(
        description=__doc__
    )
    parser.add_argument(
        'inputfile',
        default=None,
        help='input file'
    )
    parser.add_argument(
        'outputfile',
        default='output.vtk',
        nargs='?',
        help='output file'
    )
    parser.add_argument(
        '-l','--label',
        default=None,
        help='text label of vessel tree. f.e. "porta" or "hepatic_veins". \
        First label is used if it is set to None'
    )
    parser.add_argument(
        '-d', '--debug', action='store_true',
        help='Debug mode')
    args = parser.parse_args()

    if args.debug:
        ch.setLevel(logging.DEBUG)

    import imtools.gen_vtk_tree
    imtools.gen_vtk_tree.vt_file_2_vtk_file(args.inputfile, args.outputfile, args.label)


if __name__ == "__main__":
    main()



#! /usr/bin/python
# -*- coding: utf-8 -*-


import sys
import os.path

path_to_script = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(path_to_script, "../extern/sed3"))

import numpy as np

import logging
logger = logging.getLogger(__name__)

import subprocess
import scipy
import scipy.ndimage


class SparseMatrix():
    def __init__(self, ndarray):
        self.coordinates = ndarray.nonzero()
        self.shape = ndarray.shape
        self.values = ndarray[self.coordinates]
        self.dtype = ndarray.dtype
        self.sparse = True

    def todense(self):
        dense = np.zeros(self.shape, dtype=self.dtype)
        dense[self.coordinates[:]] = self.values
        return dense


def isSparseMatrix(obj):
    if obj.__class__.__name__ == 'SparseMatrix':
        return True
    else:
        return False


# import sed3

def manualcrop(data):  # pragma: no cover

    try:
        from pysegbase import seed_editor_qt
    except:
        logger.warning("Deprecated of pyseg_base as submodule")
        import seed_editor_qt

    pyed = seed_editor_qt.QTSeedEditor(data, mode='crop')
    pyed.exec_()
    # pyed = sed3.sed3(data)
    # pyed.show()
    nzs = pyed.seeds.nonzero()
    crinfo = [
        [np.min(nzs[0]), np.max(nzs[0])],
        [np.min(nzs[1]), np.max(nzs[1])],
        [np.min(nzs[2]), np.max(nzs[2])],
        ]
    data = crop(data, crinfo)
    return data, crinfo


def crop(data, crinfo):
    """
    Crop the data.

    crop(data, crinfo)

    :param crinfo: min and max for each axis

    """
    crinfo = fix_crinfo(crinfo)
    return data[
        __int_or_none(crinfo[0][0]):__int_or_none(crinfo[0][1]),
        __int_or_none(crinfo[1][0]):__int_or_none(crinfo[1][1]),
        __int_or_none(crinfo[2][0]):__int_or_none(crinfo[2][1])
        ]


def __int_or_none(number):
    if number is not None:
        number = int(number)
    return number


def combinecrinfo(crinfo1, crinfo2):
    """
    Combine two crinfos. First used is crinfo1, second used is crinfo2.
    """
    crinfo1 = fix_crinfo(crinfo1)
    crinfo2 = fix_crinfo(crinfo2)

    crinfo = [
        [crinfo1[0][0] + crinfo2[0][0], crinfo1[0][0] + crinfo2[0][1]],
        [crinfo1[1][0] + crinfo2[1][0], crinfo1[1][0] + crinfo2[1][1]],
        [crinfo1[2][0] + crinfo2[2][0], crinfo1[2][0] + crinfo2[2][1]]
        ]

    return crinfo


def crinfo_from_specific_data(data, margin):
    # hledáme automatický ořez, nonzero dá indexy
    logger.debug('crinfo')
    logger.debug(str(margin))
    nzi = np.nonzero(data)
    logger.debug(str(nzi))

    x1 = np.min(nzi[0]) - margin[0]
    x2 = np.max(nzi[0]) + margin[0] + 1
    y1 = np.min(nzi[1]) - margin[0]
    y2 = np.max(nzi[1]) + margin[0] + 1
    z1 = np.min(nzi[2]) - margin[0]
    z2 = np.max(nzi[2]) + margin[0] + 1

# ošetření mezí polí
    if x1 < 0:
        x1 = 0
    if y1 < 0:
        y1 = 0
    if z1 < 0:
        z1 = 0

    if x2 > data.shape[0]:
        x2 = data.shape[0] - 1
    if y2 > data.shape[1]:
        y2 = data.shape[1] - 1
    if z2 > data.shape[2]:
        z2 = data.shape[2] - 1

# ořez
    crinfo = [[x1, x2], [y1, y2], [z1, z2]]
    return crinfo


def uncrop(data, crinfo, orig_shape, resize=False):

    crinfo = fix_crinfo(crinfo)
    data_out = np.zeros(orig_shape, dtype=data.dtype)

    # print 'uncrop ', crinfo
    # print orig_shape
    # print data.shape
    if resize:
        data = resize_to_shape(data, crinfo[:, 1] - crinfo[:, 0])

    startx = np.round(crinfo[0][0]).astype(int)
    starty = np.round(crinfo[1][0]).astype(int)
    startz = np.round(crinfo[2][0]).astype(int)

    data_out[
        # np.round(crinfo[0][0]).astype(int):np.round(crinfo[0][1]).astype(int)+1,
        # np.round(crinfo[1][0]).astype(int):np.round(crinfo[1][1]).astype(int)+1,
        # np.round(crinfo[2][0]).astype(int):np.round(crinfo[2][1]).astype(int)+1
        startx:startx + data.shape[0],
        starty:starty + data.shape[1],
        startz:startz + data.shape[2]
        ] = data

    return data_out


def fix_crinfo(crinfo, to='axis'):
    """
    Function recognize order of crinfo and convert it to proper format.
    """

    crinfo = np.asarray(crinfo)
    if crinfo.shape[0] == 2:
        crinfo = crinfo.T

    return crinfo


def getVersionString():
    """
    Function return string with version information.
    It is performed by use one of three procedures: git describe,
    file in .git dir and file __VERSION__.
    """
    version_string = None
    try:
        version_string = subprocess.check_output(['git', 'describe'])
    except:
        logger.warning('Command "git describe" is not working')

    if version_string == None:  # noqa
        try:
            path_to_version = os.path.join(path_to_script,
                                           '../.git/refs/heads/master')
            with file(path_to_version) as f:
                version_string = f.read()
        except:
            logger.warning('Problem with reading file ".git/refs/heads/master"')

    if version_string == None:  # noqa
        try:
            path_to_version = os.path.join(path_to_script, '../__VERSION__')
            with file(path_to_version) as f:
                version_string = f.read()
            path_to_version = path_to_version + \
                '  version number is created manually'

        except:
            logger.warning('Problem with reading file "__VERSION__"')

    # if version_string is None:
    #     version_string = ""
    return version_string


def get_one_biggest_object(data):
    """ Return biggest object """
    lab, num = scipy.ndimage.label(data)
    # print ("bum = "+str(num))

    maxlab = max_area_index(lab, num)

    data = (lab == maxlab)
    return data


def max_area_index(labels, num):
    """
    Return index of maxmum labeled area
    """
    mx = 0
    mxi = -1
    for l in range(1, num + 1):
        mxtmp = np.sum(labels == l)
        if mxtmp > mx:
            mx = mxtmp
            mxi = l

    return mxi


def resize_to_mm(data3d, voxelsize_mm, new_voxelsize_mm, mode='nearest'):
    """
    Function can resize data3d or segmentation to specifed voxelsize_mm
    :new_voxelsize_mm: requested voxelsize. List of 3 numbers, also 
        can be a string 'orig', 'orgi*2' and 'orgi*4'.
        
    :voxelsize_mm: size of voxel
    :mode: default is 'nearest'
    """

    if new_voxelsize_mm == 'orig':
        new_voxelsize_mm = np.array(voxelsize_mm)

    elif new_voxelsize_mm == 'orig*2':
        new_voxelsize_mm = np.array(voxelsize_mm) * 2
    elif new_voxelsize_mm == 'orig*4':
        new_voxelsize_mm = np.array(voxelsize_mm) * 4
        # vx_size = np.array(metadata['voxelsize_mm']) * 4

    zoom = voxelsize_mm / (1.0 * np.array(new_voxelsize_mm))
    data3d_res = scipy.ndimage.zoom(
        data3d,
        zoom,
        mode=mode,
        order=1
    ).astype(data3d.dtype)
    return data3d_res


def resize_to_shape(*pars, **params):
    import misc
    return misc.resize_to_shape(*pars, **params)

    # nzoom = shape / np.array(segmentation.shape).astype(np.double)
    #
    # segm_orig_scale = scipy.ndimage.zoom(
    #     segmentation,
    #     nzoom,
    #     mode='nearest',
    #     order=0
    # ).astype(segmentation.dtype)
    #
    # return segm_orig_scale



#! /usr/bin/python
# -*- coding: utf-8 -*-
"""
Experiment s \"barevným\" modelem jater.

Pro spuštění zkuste help:

python experiments/20130919_liver_statistics.py --help

Měřené vlastnosti se přidávají do get_features().
Pro přidání dalších dat, editujte příslušný yaml soubor.

"""

# import funkcí z jiného adresáře
import sys
import os.path

path_to_script = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(path_to_script, "../src/"))
sys.path.append(os.path.join(path_to_script, "../extern/pyseg_base/src"))
sys.path.append(os.path.join(path_to_script,
                             "../extern/sed3/"))
sys.path.append(os.path.join(path_to_script,
                             "../extern/lbp/"))
#sys.path.append(os.path.join(path_to_script, "../extern/"))
#import featurevector

import logging
logger = logging.getLogger(__name__)


#import apdb
#  apdb.set_trace();
#import scipy.io
import numpy as np
#import scipy
#from scipy import sparse
#import traceback
import itertools

# ----------------- my scripts --------
import sed3
#import dcmreaddata1 as dcmr
#import dcmreaddata as dcmr
import argparse
#import sed3

import misc
import datareader
import matplotlib.pyplot as plt
import experiments


def feat_hist_by_segmentation(data3d_orig, data3d_seg, visualization=True):
    bins = range(-1024, 1024, 1)
    bins = range(-512, 512, 1)
    hist1, bin_edges1 = np.histogram(data3d_orig[data3d_seg > 0], bins=bins)
    hist2, bin_edges2 = np.histogram(data3d_orig[data3d_seg <= 0], bins=bins)
    #import pdb; pdb.set_trace()
    if visualization:
        plt_liver = plt.step(bin_edges1[1:], hist1)
        plt_rest = plt.step(bin_edges2[1:], hist2)
        plt.legend([plt_liver, plt_rest], ['Liver', 'Other tissue'])
        #plt.plot(bin_edges1[1:], hist1, bin_edges2[1:], hist2)
        plt.show()
    fv_hist = {'hist1': hist1,
               'hist2': hist2,
               'bins': bins
               }
    return fv_hist


def feat_hist(data3d_orig):
    bins = range(-1024, 1024, 1)
    bins = range(-512, 512, 1)
    bins = range(-512, 512, 10)
    bins = range(-512, 512, 64)
    bins = range(-512, 512, 100)
    hist1, bin_edges1 = np.histogram(data3d_orig, bins=bins)
    return hist1



def get_features(data3d_orig, data3d_seg, feature_fcn, visualization=True):
    u"""
    Sem doplníme všechny naše měření.

    Pro ukázku jsem vytvořil měření
    histogramu.
    data3d_orig: CT data3d_orig
    data3d_seg: jedničky tam, kde jsou játra

    """

    featur = feature_fcn(data3d_orig)
    #featur = {}
    #featur['hist'] = feat_hist(data3d_orig, visualization)
    #featur['lbp'] = lbp(data3d_orig, data3d_seg, visualization)

    return featur


def sample_input_data():
    inputdata = {
        'basedir': '/home/mjirik/data/medical/',
        'data': [
            {'sliverseg': 'data_orig/sliver07/training-part1/liver-seg001.mhd', 'sliverorig': 'data_orig/sliver07/training-part1/liver-orig001.mhd'},
            {'sliverseg': 'data_orig/sliver07/training-part1/liver-seg002.mhd', 'sliverorig': 'data_orig/sliver07/training-part1/liver-orig002.mhd'},
        ]
    }

    sample_data_file = os.path.join(path_to_script,
                                    "20130919_liver_statistics.yaml")
    #print sample_data_file, path_to_script
    misc.obj_to_file(inputdata, sample_data_file, filetype='yaml')

#def voe_metric(vol1, vol2, voxelsize_mm):


def write_csv(data, filename="20130919_liver_statistics.yaml"):
    import csv
    with open(filename, 'wb') as csvfile:
        spamwriter = csv.writer(csvfile,
                                delimiter=';',
                                quotechar='"',
                                quoting=csv.QUOTE_MINIMAL
                                )
        for label in data:
            spamwriter.writerow([label] + data[label])
            #spamwriter.writerow(['Spam', 'Lovely Spam', 'Wonderful Spam'])


#def training_dataset_prepare
def read_data_orig_and_seg(inputdata, i):
    """ Loads data_orig and data_seg from yaml file
    """

    reader = datareader.DataReader()
    data3d_a_path = os.path.join(inputdata['basedir'],
                                 inputdata['data'][i]['sliverseg'])
    data3d_a, metadata_a = reader.Get3DData(data3d_a_path)

    data3d_b_path = os.path.join(inputdata['basedir'],
                                 inputdata['data'][i]['sliverorig'])
    data3d_b, metadata_b = reader.Get3DData(data3d_b_path)

    #import pdb; pdb.set_trace()
    data3d_seg = (data3d_a > 0).astype(np.int8)
    data3d_orig = data3d_b

    return data3d_orig, data3d_seg


def one_experiment_setting_for_whole_dataset(inputdata, tile_shape,
                                             feature_fcn, classif_fcn, train,
                                             visualization=False):
    fvall = []
    fv_tiles = []
    indata_len = len(inputdata['data'])
    indata_len = 3

    for i in range(0, indata_len):
        data3d_orig, data3d_seg = read_data_orig_and_seg(inputdata, i)

        feat_hist_by_segmentation(data3d_orig, data3d_seg, visualization)

        if visualization:
            pyed = sed3.sed3(data3d_orig,
                                                 contour=data3d_seg)
            pyed.show()

            #import pdb; pdb.set_trace()
        #fvall.insert(i, get_features(
        #    data3d_orig,
            #ltl = (labels_train_lin_float * 10).astype(np.int8)
            #labels_train = arrange_to_tiled_data(cidxs, tile_shape,
            #                                     d_shp, ltl)

            #pyed = sed3.sed3(labels_train, contour=labels)

# @TODO vracet něco inteligentního, fvall je prázdný
    return fvall


def make_product_list(list_of_feature_fcn, list_of_classifiers):
#   TODO work with list_of_feature_fcn and list_of_classifiers
    featrs_plus_classifs = itertools.product(list_of_feature_fcn,
                                             list_of_classifiers)
    return featrs_plus_classifs


def experiment(path_to_yaml, featrs_plus_classifs,
               tile_shape, visualization=False, train=False):

    inputdata = misc.obj_from_file(path_to_yaml, filetype='yaml')

    #import ipdb; ipdb.set_trace()  # noqa BREAKPOINT

    results = []

    for fpc in featrs_plus_classifs:
        feature_fcn = fpc[0]
        classif_fcn = fpc[1]

        fvall = one_experiment_setting_for_whole_dataset(
            inputdata, tile_shape,
            feature_fcn, classif_fcn, train, visualization)

        result = {'params': str(fpc), 'fvall': fvall}
        results.append(result)
        print results

    return results


def main():

    #logger = logging.getLogger(__name__)
    logger = logging.getLogger()

    logger.setLevel(logging.WARNING)
    ch = logging.StreamHandler()
    logger.addHandler(ch)

    #logger.debug('input params')

    parser = argparse.ArgumentParser(
        description='Compute features on liver and other tissue.')
    parser.add_argument('-si', '--sampleInput', action='store_true',
                        help='generate sample intput data', default=False)
    parser.add_argument('-v', '--visualization',  action='store_true',
                        help='Turn on visualization', default=False)
    parser.add_argument('-i', '--input', help='input yaml file',
                        default="20130919_liver_statistics.yaml")
    parser.add_argument('-o', '--output', help='output file',
                        default="20130919_liver_statistics_results.pkl")
    parser.add_argument('-t', '--train', help='Training', default=False,
                        action='store_true'
                        )
    args = parser.parse_args()

    if args.sampleInput:
        sample_input_data()
    # input parser
    #path_to_yaml = os.path.join(path_to_script, args.input)
    path_to_yaml = args.input

    #write_csv(fvall)
    list_of_feature_fcn = [feat_hist]
    from sklearn import svm
    from sklearn.naive_bayes import GaussianNB

    list_of_classifiers = [svm.SVC, GaussianNB]
    tile_shape = [1, 100, 100]
    featrs_plus_classifs = make_product_list(list_of_feature_fcn,
                                             list_of_classifiers)

    result = experiment(path_to_yaml, featrs_plus_classifs,
                        tile_shape=tile_shape,
                        visualization=args.visualization, train=args.train)

# Ukládání výsledku do souboru
    output_file = os.path.join(path_to_script, args.output)
    misc.obj_to_file(result, output_file, filetype='pickle')

if __name__ == "__main__":
    main()



import numpy as np


def translate(point, vector, length=None):
    vector = np.array(vector)
    if length is not None:
        vector = length * vector / np.linalg.norm(vector)
    return (np.array(point) + vector).tolist()


def circle(center, perp_vect, radius, element_number=10):
    """
    Function computed the circle points. No drawing.
    perp_vect is vector perpendicular to plane of circle
    """
    # tl = [0, 0.2, 0.4, 0.6, 0.8]
    tl = np.linspace(0, 1, element_number)

    # vector form center to edge of circle
    # u is a unit vector from the centre of the circle to any point on the
    # circumference

    # normalized perpendicular vector
    n = perp_vect / np.linalg.norm(perp_vect)

    # normalized vector from the centre to point on the circumference
    u = perpendicular_vector(n)
    u = u / np.linalg.norm(u)

    pts = []

    for t in tl:
        # u = np.array([0, 1, 0])
        # n = np.array([1, 0, 0])
        pt = radius * np.cos(t * 2 * np.pi) * u +\
            radius * np.sin(t * 2 * np.pi) * np.cross(u, n) +\
            center

        pt = pt.tolist()
        pts.append(pt)

    return pts


def perpendicular_vector(v):
    r""" Finds an arbitrary perpendicular vector to *v*."""
    if v[1] == 0 and v[2] == 0:
        if v[0] == 0:
            raise ValueError('zero vector')
        else:
            return np.cross(v, [0, 1, 0])
    return np.cross(v, [1, 0, 0])


def cylinder_circles(nodeA, nodeB, radius, element_number=10):
    """
    Return list of two circles with defined parameters.
    """

    vector = (np.array(nodeA) - np.array(nodeB)).tolist()
    ptsA = circle(nodeA, vector, radius, element_number)
    ptsB = circle(nodeB, vector, radius, element_number)

    return ptsA, ptsB



#! /usr/bin/python
# -*- coding: utf-8 -*-
"""
Evaluation of liver volume error inspired by Sliver07.
Input is YAML file with dataset metadata.

Function compare_volumes can be used to compare two data.

Compute Sliver07 score::

eval_data = lisa.volumetry_evaluation.compare_volumes(
    ndimage_3d_segmentation_1,
    ndimage_3d_segmentation_2,
    voxelsize_mm)

score = lisa.volumetry_evaluation.sliver_score_one_couple(eval_data)

"""

# import funkcí z jiného adresáře
import sys
import os.path

path_to_script = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(path_to_script, "../src/"))
sys.path.append(os.path.join(path_to_script, "../extern/pyseg_base/src"))
sys.path.append(os.path.join(path_to_script,
                             "../extern/sed3/"))
# sys.path.append(os.path.join(path_to_script, "../extern/"))
# import featurevector

import logging

logger = logging.getLogger(__name__)


# import apdb
# apdb.set_trace();
# import scipy.io
import numpy as np
import scipy
# from scipy import sparse
# import traceback

# ----------------- my scripts --------
import sed3
# import dcmreaddata1 as dcmr
# import dcmreaddata as dcmr
import argparse

# import segmentation
import qmisc
import misc
# import organ_segmentation
# import experiments
from io3d import datareader

# okraj v pixelech využitý pro snížení výpočetní náročnosti během vyhodnocování
# AvgD
CROP_MARGIN = [20]


def evaluate_and_write_to_file(
        inputYamlFile,
        directoryPklz,
        directorySliver,
        outputfile,
        visualization,
        return_dir_lists=False,
        special_evaluation_function=None,
        return_all_data=False
):
    """
    Function computes yaml file (if there are given input sliver and pklz
    directories). Based on yaml file are compared sliver segmentations and
    our pklz files.
    """
    dirlists = None
    if (directoryPklz is not None) and (directorySliver is not None):
        dirlists = generate_input_yaml(
            directorySliver,
            directoryPklz,
            yaml_filename=inputYamlFile,
            return_dir_lists=return_dir_lists
        )

    # input parser
    data_file = inputYamlFile
    inputdata = misc.obj_from_file(data_file, filetype='yaml')

    evaluation_all = eval_all_from_dataset_metadata(
        inputdata, visualization,
        special_evaluation_function=special_evaluation_function
    )

    logger.debug(str(evaluation_all))
    logger.debug('eval all')

    logger.debug(make_sum(evaluation_all))
    write_csv(evaluation_all, filename=outputfile + '.csv')
    misc.obj_to_file(evaluation_all, outputfile + '.pkl', filetype='pkl')

    retval = []

    if return_dir_lists:
        retval.append(dirlists)
    if return_all_data:
        retval.append(evaluation_all)
    if len(retval) > 0:
        return retval

        # import pdb; pdb.set_trace()

        # volume
        # volume_mm3 = np.sum(oseg.segmentation > 0) * np.prod(oseg.voxelsize_mm)

        # pyed = sed3.sed3(oseg.data3d, contour =
        # oseg.segmentation)
        # pyed.show()

    # if args.show_output:
    # oseg.show_output()
    #
    # savestring = raw_input('Save output data? (y/n): ')
    # sn = int(snstring)
    # if savestring in ['Y', 'y']:
    #
    #        data = oseg.export()
    #
    #        misc.obj_to_file(data, "organ.pkl", filetype='pickle')
    #        misc.obj_to_file(oseg.get_ipars(), 'ipars.pkl', filetype='pickle')
    # output = segmentation.vesselSegmentation(oseg.data3d,
    # oseg.orig_segmentation)


def eval_all_from_dataset_metadata(inputdata, visualization=False,
                                   special_evaluation_function=None):
    """
    set metadata
    """
    evaluation_all = {
        'file1': [],
        'file2': [],
        'volume1_mm3': [],
        'volume2_mm3': [],
        'err1_mm3': [],
        'err2_mm3': [],
        'err1_percent': [],
        'err2_percent': [],
        'voe': [],
        'vd': [],
        'avgd': [],
        'rmsd': [],
        'processing_time': [],
        'organ_interactivity_counter': [],
        'maxd': []

    }
    for i in range(0, len(inputdata['data'])):

        reader = datareader.DataReader()
        data3d_a_path = os.path.join(inputdata['basedir'],
                                     inputdata['data'][i]['sliverseg'])
        data3d_a, metadata_a = reader.Get3DData(data3d_a_path)
        print "inputdata ", inputdata['data'][i].values()
        try:
            # if there is defined overlay
            data3d_a = reader.GetOverlay()[inputdata['data'][i][
                'overlay_number']]
            logger.info('overlay loaded')
            print 'overlay loaded'
        except:
            logger.debug("overlay not loaded")
            pass

        logger.debug('data A shape ' + str(data3d_a.shape))

        data3d_b_path = os.path.join(inputdata['basedir'],
                                     inputdata['data'][i]['ourseg'])
        obj_b = misc.obj_from_file(data3d_b_path, filetype='pickle')
        # data_b, metadata_b = reader.Get3DData(data3d_b_path)

        if 'crinfo' in obj_b.keys():
            data3d_b = qmisc.uncrop(obj_b['segmentation'],
                                    obj_b['crinfo'], data3d_a.shape)
        else:
            data3d_b = obj_b['segmentation']

        # import pdb; pdb.set_trace()
        # data3d_a = (data3d_a > 1024).astype(np.int8)
        data3d_a = (data3d_a > 0).astype(np.int8)
        data3d_b = (data3d_b > 0).astype(np.int8)

        if visualization:
            pyed = sed3.sed3(data3d_a,  # + (4 * data3d_b)
                             contour=data3d_b)
            pyed.show()

        evaluation_one = compare_volumes(data3d_a, data3d_b,
                                         metadata_a['voxelsize_mm'])
        if special_evaluation_function is not None:
            evaluation_one.update(
                special_evaluation_function(
                    data3d_a, data3d_b, metadata_a['voxelsize_mm']
                ))
        evaluation_all['file1'].append(data3d_a_path)
        evaluation_all['file2'].append(data3d_b_path)
        for key in evaluation_one.keys():
            if key not in evaluation_all.keys():
                evaluation_all[key] = []

            evaluation_all[key].append(evaluation_one[key])
        # evaluation_all['volume1_mm3'].append(evaluation_one['volume1_mm3'])
        # evaluation_all['volume2_mm3'].append(evaluation_one['volume2_mm3'])
        # evaluation_all['err1_mm3'].append(evaluation_one['err1_mm3'])
        # evaluation_all['err2_mm3'].append(evaluation_one['err2_mm3'])
        # evaluation_all['err1_percent'].append(evaluation_one['err1_percent'])
        # evaluation_all['err2_percent'].append(evaluation_one['err2_percent'])
        # evaluation_all['voe'].append(evaluation_one['voe'])
        # evaluation_all['vd'].append(evaluation_one['vd'])
        # evaluation_all['avgd'].append(evaluation_one['avgd'])
        # evaluation_all['rmsd'].append(evaluation_one['rmsd'])
        # evaluation_all['maxd'].append(evaluation_one['maxd'])
        if 'processing_time' in obj_b.keys():
            # this is only for compatibility with march2014 data
            processing_time = obj_b['processing_time']
            organ_interactivity_counter = obj_b['organ_interactivity_counter']
        else:
            try:
                processing_time = obj_b['processing_information']['organ_segmentation']['processing_time']  # noqa
                organ_interactivity_counter = obj_b['processing_information']['organ_segmentation'][
                    'organ_interactivity_counter']  # noqa
            except:
                processing_time = 0
                organ_interactivity_counter = 0
        evaluation_all['processing_time'].append(processing_time)
        evaluation_all['organ_interactivity_counter'].append(
            organ_interactivity_counter)

    return evaluation_all


def compare_volumes_boundingbox(vol1, vol2, voxelsize_mm):
    import qmisc

    crinfo = qmisc.crinfo_from_specific_data(vol1, [20, 20, 20])
    vol1[
    crinfo[0][0]:crinfo[0][1],
    crinfo[1][0]:crinfo[1][1],
    crinfo[2][0]:crinfo[2][1]
    ] = 1

    volume1 = np.sum(vol1 > 0)
    volume2 = np.sum(vol2 > 0)
    volume1_mm3 = volume1 * np.prod(voxelsize_mm)
    volume2_mm3 = volume2 * np.prod(voxelsize_mm)
    volume_avg_mm3 = (volume1_mm3 + volume2_mm3) * 0.5

    df = vol1 - vol2
    df1 = np.sum(df == 1) * np.prod(voxelsize_mm)
    df2 = np.sum(df == -1) * np.prod(voxelsize_mm)

    evaluation = {
        'box_err1_mm3': df1,
        'box_err2_mm3': df2,
        'box_err1_percent': df1 / volume_avg_mm3 * 100,
        'box_err2_percent': df2 / volume_avg_mm3 * 100,
    }
    return evaluation

def compare_volumes_sliver(vol1, vol2, voxelsize_mm, use_logger=False):
    """
    Computes statistics from similarity of vol1 and vol2. Return the same as
    compare_volumes with additional information with sliver score
    :param vol1:
    :param vol2:
    :param voxelsize_mm:
    :param use_logger:
    :return:
    """

    evaluation = compare_volumes(vol1, vol2, voxelsize_mm, use_logger)
    sliver_evaluation = sliver_score_one_couple(
        evaluation,
        keys=[
            'sliver_vd_pts',
            'sliver_voe_pts',
            'sliver_avgd_pts',
            'sliver_rmsd_pts',
            'sliver_maxd_pts'
        ])
    overall = sliver_overall_score_for_one_couple(sliver_evaluation)

    evaluation.update(sliver_evaluation)
    evaluation['sliver_overall_pts'] = overall
    return evaluation

def compare_volumes(vol1, vol2, voxelsize_mm, use_logger=False):
    """
    computes metrics, no sliver computed here, see compare_volumes_sliver

    vol1: reference
    vol2: segmentation
    """
    volume1 = np.sum(vol1 > 0)
    volume2 = np.sum(vol2 > 0)
    volume1_mm3 = volume1 * np.prod(voxelsize_mm)
    volume2_mm3 = volume2 * np.prod(voxelsize_mm)
    volume_avg_mm3 = (volume1_mm3 + volume2_mm3) * 0.5
    if use_logger:
        logger.debug('vol1 [mm3]: ' + str(volume1_mm3))
        logger.debug('vol2 [mm3]: ' + str(volume2_mm3))

    df = vol1 - vol2
    df1 = np.sum(df == 1) * np.prod(voxelsize_mm)
    df2 = np.sum(df == -1) * np.prod(voxelsize_mm)

    if use_logger:
        logger.debug('err- [mm3]: ' + str(df1) + ' err- [%]: '
                     + str(df1 / volume_avg_mm3 * 100))
        logger.debug('err+ [mm3]: ' + str(df2) + ' err+ [%]: '
                     + str(df2 / volume_avg_mm3 * 100))

    # VOE[%]
    intersection = np.sum(df != 0).astype(np.float)
    union = (np.sum(vol1 > 0) + np.sum(vol2 > 0)).astype(float)
    voe = 100 * (intersection / union)
    if use_logger:
        logger.debug('VOE [%]' + str(voe))

    # VD[%]
    vd = 100 * (volume2 - volume1).astype(float) / volume1.astype(float)
    if use_logger:
        logger.debug('VD [%]' + str(vd))
    # import pdb; pdb.set_trace()

    # pyed = sed3.sed3(vol1, contour=vol2)
    # pyed.show()

    # get_border(vol1)
    avgd, rmsd, maxd = distance_matrics(vol1, vol2, voxelsize_mm)
    if use_logger:
        logger.debug('AvgD [mm]' + str(avgd))
        logger.debug('RMSD [mm]' + str(rmsd))
        logger.debug('MaxD [mm]' + str(maxd))
    evaluation = {
        'volume1_mm3': volume1_mm3,
        'volume2_mm3': volume2_mm3,
        'err1_mm3': df1,
        'err2_mm3': df2,
        'err1_percent': df1 / volume_avg_mm3 * 100,
        'err2_percent': df2 / volume_avg_mm3 * 100,
        'vd': vd,
        'voe': voe,
        'avgd': avgd,
        'rmsd': rmsd,
        'maxd': maxd
    }

    return evaluation


def distance_matrics(vol1, vol2, voxelsize_mm):
    # crop data to reduce comutation time
    crinfo = qmisc.crinfo_from_specific_data(vol1 + vol2, CROP_MARGIN)
    logger.debug(str(crinfo) + ' m1 ' + str(np.max(vol1)) +
                 ' m2 ' + str(np.min(vol2)))
    logger.debug("crinfo " + str(crinfo))
    vol1 = qmisc.crop(vol1, crinfo)
    vol2 = qmisc.crop(vol2, crinfo)

    border1 = _get_border(vol1)
    border2 = _get_border(vol2)

    # pyed = sed3.sed3(vol1, contour=vol1)
    # pyed.show()
    b1dst = scipy.ndimage.morphology.distance_transform_edt(
        1 - border1,
        sampling=voxelsize_mm
    )
    b2dst = scipy.ndimage.morphology.distance_transform_edt(
        1 - border2,
        sampling=voxelsize_mm
    )

    dst_b1_to_b2 = border2 * b1dst
    dst_b2_to_b1 = border1 * b2dst
    dst_12 = dst_b1_to_b2[border2]
    dst_21 = dst_b2_to_b1[border1]
    dst_both = np.append(dst_12, dst_21)

    # sum_d12 = np.sum(dst_12)
    # sum_d21 = np.sum(dst_21)
    # len_d12 = len(dst_12)
    # len_d21 = len(dst_21)
    # import ipdb; ipdb.set_trace() # BREAKPOINT
    # pyed = sed3.sed3(dst_b1_to_b2, contour=vol1)
    # pyed.show()
    # print np.nonzero(border1)
    # avgd = np.average(dst_b1_to_b2[np.nonzero(border2)])
    # import ipdb; ipdb.set_trace() #  noqa BREAKPOINT
    # avgd = (sum_d21 + sum_d12)/float(len_d21 + len_d12)
    # rmsd = ((np.sum(dst_12**2) + dst_21**2)/float(len_d21 + len_d12))**0.5
    avgd = np.average(dst_both)

    # there is not clear what is correct
    # rmsd = np.average(dst_both ** 2)**0.5
    # rmsd = (np.average(dst_12) + np.average(dst_21))**0.5
    rmsd = np.average(dst_both ** 2)
    # rmsd = np.average(dst_b1_to_b2[border2] ** 2)
    maxd = max(np.max(dst_b1_to_b2), np.max(dst_b2_to_b1))
    # old
    # avgd = np.average(dst_b1_to_b2[border2])
    # rmsd = np.average(dst_b1_to_b2[border2] ** 2)
    # maxd = np.max(dst_b1_to_b2)

    return avgd, rmsd, maxd


def _get_border(image3d):
    import scipy.ndimage

    kernel = np.ones([3, 3, 3])
    conv = scipy.ndimage.convolve(image3d, kernel)
    conv[conv == 27] = 0
    conv = conv * image3d

    conv = conv > 0

    # pyed = sed3.sed3(conv, contour =
    # image3d)
    # pyed.show()

    return conv


def write_csv(data, filename='20130812_liver_volumetry.csv'):
    logger.debug(filename)
    import csv

    with open(filename, 'wb') as csvfile:
        writer = csv.writer(
            csvfile,
            delimiter=';',
            quotechar='"',
            quoting=csv.QUOTE_MINIMAL
        )
        write_sum_to_csv(data, writer)
        for label in data:
            writer.writerow([label] + data[label])
            # spamwriter.writerow(['Spam', 'Lovely Spam', 'Wonderful Spam'])


def write_sum_to_csv(evaluation, writer):
    avg, var = make_sum(evaluation)
    key = evaluation.keys()
    writer.writerow([' - '] + key)
    writer.writerow(['var'] + var)
    writer.writerow(['avg'] + avg)
    writer.writerow([])


def sliver_overall_score_for_one_couple(score):
    """
    Computes overall score:
    """
    score_list = score.values()
    overall_score = np.average(np.array(score_list))

    return overall_score


def sliverScore(measure, metric_type):
    """
    deprecated name of sliver_score. Use new sliver_score() function.
    """
    print "Deprecated function sliverScore(). Use sliver_score()."
    return sliver_score(measure, metric_type)


def sliver_score(measure, metric_type):
    """
    Based on sliver metodics
    http://sliver07.org/p7.pdf

    Slope and intercept comutations:
    https://docs.google.com/spreadsheet/ccc?key=0AkBzbxly5bqfdEJaOWJJUEh5ajVJM05YWGdaX1k5aFE#gid=0   # noqa

    """
    slope = -1
    intercept = 100

    if metric_type is 'vd':
        slope = -3.90625
    elif metric_type is 'voe':
        slope = -5.31914893617021
    elif metric_type is 'avgd':
        slope = -25
    elif metric_type is 'rmsd':
        slope = -14.7058823529412
    elif metric_type is 'maxd':
        slope = -1.31578947368421

    score = intercept + np.abs(measure) * slope

    try:
        score[score < 0] = 0
    except:
        # if score is scalar
        if score < 0:
            score = 0

    return score


def sliver_score_one_couple(
        data,
        keys=['vd', 'voe', 'avgd', 'rmsd', 'maxd']):
    """
    Convert data from compare_volumes() function.
    :param keys: set output keys
    """
    score = {
        keys[0]: sliver_score(data['vd'], 'vd'),
        keys[1]: sliver_score(data['voe'], 'voe'),
        keys[2]: sliver_score(data['avgd'], 'avgd'),
        keys[3]: sliver_score(data['rmsd'], 'rmsd'),
        keys[4]: sliver_score(data['maxd'], 'maxd'),
    }
    return score


def sliverScoreAll(data):
    """
    Computers score by Sliver07
    http://sliver07.org/p7.pdf

    input: dataset = [{'vd':[1.1, ..., 0.1], 'voe':[...], 'avgd':[...], ...}
                      {'vd':[...], 'voe':[...], ...}
                     ]
    return: scoreTotal, scoreMetrics, scoreAll
        Order of scoreMetrics is [vd, voe, avgd, rmsd, maxd]


    """

    scoreAll = []
    scoreTotal = []
    scoreMetrics = []
    for dat in data:
        score = {
            'vd': sliver_score(dat['vd'], 'vd'),
            'voe': sliver_score(dat['voe'], 'voe'),
            'avgd': sliver_score(dat['avgd'], 'avgd'),
            'rmsd': sliver_score(dat['rmsd'], 'rmsd'),
            'maxd': sliver_score(dat['maxd'], 'maxd'),
        }
        scoreAll.append(score)

        metrics = [np.mean(score['vd']),
                   np.mean(score['voe']),
                   np.mean(score['avgd']),
                   np.mean(score['rmsd']),
                   np.mean(score['maxd'])
                   ]
        scoreMetrics.append(metrics)

        total = np.mean(metrics)
        scoreTotal.append(total)

    return scoreTotal, scoreMetrics, scoreAll


def make_sum(evaluation):
    var = []
    avg = []
    for key in evaluation.keys():
        avgi = 'nan'
        vari = 'nan'
        try:
            avgi = np.average(evaluation[key])
            vari = np.var(evaluation[key])
        except Exception:
            print "problem with key: ", key
            # print evaluation[key]
        avg.append(avgi)
        var.append(vari)
    return avg, var


def main():
    # logger = logging.getLogger(__name__)
    logger = logging.getLogger()

    logger.setLevel(logging.WARNING)
    ch = logging.StreamHandler()
    logger.addHandler(ch)

    # logger.debug('input params')

    default_data_file = os.path.join(path_to_script,
                                     "20130812_liver_volumetry.yaml")

    parser = argparse.ArgumentParser(
        description='Compare two segmentation. Evaluation is similar\
        to MICCAI 2007 workshop.  Metrics are described in\
        www.sliver07.com/p7.pdf')
    parser.add_argument('-d', '--debug', action='store_true',
                        help='run in debug mode', default=False)
    parser.add_argument('-si', '--sampleInput', action='store_true',
                        help='generate sample intput data', default=False)
    parser.add_argument('-v', '--visualization', action='store_true',
                        help='Turn on visualization', default=False)
    parser.add_argument('-y', '--inputYamlFile', help='input yaml file',
                        default=default_data_file)
    parser.add_argument('-ds', '--directorySliver',
                        help='input SLiver directory. If this and\
                        directoryPklz is not None, yaml file is generated',
                        default=None)
    parser.add_argument('-dp', '--directoryPklz', help='input pklz directory',
                        default=None)
    parser.add_argument('-o', '--outputfile',
                        help='output file without extension',
                        default='20130812_liver_volumetry')
    args = parser.parse_args()

    if args.debug:
        logger.setLevel(logging.DEBUG)
        logger.debug('debug mode logging')

    if args.sampleInput:
        sample_input_data()

    evaluate_and_write_to_file(
        args.inputYamlFile,
        args.directoryPklz,
        args.directorySliver,
        args.outputfile,
        args.visualization
    )


def generate_input_yaml(sliver_dir, pklz_dir,
                        sliver_ext='*seg0*.mhd', pklz_ext='*0*.pklz',
                        yaml_filename=None,
                        return_dir_lists=False
                        ):
    """
    Function pair files from different directory by numer in format g0XX.
    It is ok for seg001 and orig001 too.
    If there is given some yaml_filename, it is created.
    """
    import glob
    import re

    if not os.path.exists(sliver_dir):
        raise IOError("Directory with reference data does not exists")
    if not os.path.exists(pklz_dir):
        raise IOError("Directory with input pklz data does not exists")

    onlyfiles1 = glob.glob(os.path.join(sliver_dir, sliver_ext))
    onlyfiles2 = glob.glob(os.path.join(pklz_dir, pklz_ext))
    onlyfiles1.sort()
    onlyfiles2.sort()
    if len(onlyfiles1) == 0:
        raise IOError("Directory with reference data appears to be empty")
    if len(onlyfiles2) == 0:
        raise IOError("Directory with pklz data appears to be empty")

    logger.debug('sliver files \n' + str(onlyfiles1))
    logger.debug('pklz files \n' + str(onlyfiles2))

    data = []
    for flns in onlyfiles1:
        base, flnsh = os.path.split(os.path.normpath(flns))
        pattern = re.search('(g0[0-9]{2})', flnsh)
        if pattern:
            pattern = pattern.group(1)
        logger.debug('pattern1 ' + pattern)

        for flnp in onlyfiles2:
            base, flnph = os.path.split(os.path.normpath(flnp))
            pt = re.match('.*' + pattern + '.*', flnph)
            if pt:
                data.append({
                    'sliverseg': flns,
                    'ourseg': flnp
                })

    inputdata = {
        'basedir': '',
        'data': data
    }

    retval = []

    if yaml_filename is None:
        retval.append(inputdata)
    else:
        misc.obj_to_file(inputdata, yaml_filename, filetype='yaml')

    if return_dir_lists:
        retval.append(onlyfiles1)
        retval.append(onlyfiles2)

    if len(retval) > 1:
        return tuple(retval)
    elif len(retval) == 1:
        return retval[0]


def sample_input_data():
    inputdata = {'basedir': '/home/mjirik/data/medical/',  # noqa
                 'data': [
                     {'sliverseg': 'data_orig/sliver07/training-part1/liver-seg001.mhd',
                      'ourseg': 'data_processed/organ_small-liver-orig001.mhd.pkl'},  # noqa
                     {'sliverseg': 'data_orig/sliver07/training-part1/liver-seg002.mhd',
                      'ourseg': 'data_processed/organ_small-liver-orig002.mhd.pkl'},  # noqa
                     {'sliverseg': 'data_orig/sliver07/training-part1/liver-seg003.mhd',
                      'ourseg': 'data_processed/organ_small-liver-orig003.mhd.pkl'},  # noqa
                     {'sliverseg': 'data_orig/sliver07/training-part1/liver-seg004.mhd',
                      'ourseg': 'data_processed/organ_small-liver-orig004.mhd.pkl'},  # noqa
                     {'sliverseg': 'data_orig/sliver07/training-part1/liver-seg005.mhd',
                      'ourseg': 'data_processed/organ_small-liver-orig005.mhd.pkl'},  # noqa
                     {'sliverseg': 'data_orig/sliver07/training-part2/liver-seg006.mhd',
                      'ourseg': 'data_processed/organ_small-liver-orig006.mhd.pkl'},  # noqa
                     {'sliverseg': 'data_orig/sliver07/training-part2/liver-seg007.mhd',
                      'ourseg': 'data_processed/organ_small-liver-orig007.mhd.pkl'},  # noqa
                     {'sliverseg': 'data_orig/sliver07/training-part2/liver-seg008.mhd',
                      'ourseg': 'data_processed/organ_small-liver-orig008.mhd.pkl'},  # noqa
                     {'sliverseg': 'data_orig/sliver07/training-part2/liver-seg009.mhd',
                      'ourseg': 'data_processed/organ_small-liver-orig009.mhd.pkl'},  # noqa
                 ]
                 }

    sample_data_file = os.path.join(path_to_script,
                                    "20130812_liver_volumetry_sample.yaml")
    # print sample_data_file, path_to_script
    misc.obj_to_file(inputdata, sample_data_file, filetype='yaml')

# def voe_metric(vol1, vol2, voxelsize_mm):


if __name__ == "__main__":
    main()



import numpy as nm

def set_nodemtx(mtx, idxs, etype):

    dim = len(idxs)
    if dim == 2:
        ix, iy = idxs

        if etype == 'q':
            mtx[ix,iy] += 1
            mtx[ix + 1,iy] += 1
            mtx[ix + 1,iy + 1] += 1
            mtx[ix,iy + 1] += 1

        elif etype == 't':
            mtx[ix,iy] += 2
            mtx[ix + 1,iy] += 1
            mtx[ix + 1,iy + 1] += 2
            mtx[ix,iy + 1] += 1

    elif dim == 3:
        ix, iy, iz = idxs

        if etype == 'q':
            mtx[ix,iy,iz] += 1
            mtx[ix + 1,iy,iz] += 1
            mtx[ix + 1,iy + 1,iz] += 1
            mtx[ix,iy + 1,iz] += 1
            mtx[ix,iy,iz + 1] += 1
            mtx[ix + 1,iy,iz + 1] += 1
            mtx[ix + 1,iy + 1,iz + 1] += 1
            mtx[ix,iy + 1,iz + 1] += 1

        elif etype == 't':
            mtx[ix,iy,iz] += 6
            mtx[ix + 1,iy,iz] += 2
            mtx[ix + 1,iy + 1,iz] += 2
            mtx[ix,iy + 1,iz] += 2
            mtx[ix,iy,iz + 1] += 2
            mtx[ix + 1,iy,iz + 1] += 2
            mtx[ix + 1,iy + 1,iz + 1] += 6
            mtx[ix,iy + 1,iz + 1] += 2

    else:
        msg = 'incorrect voxel dimension! (%d)' % dim
        raise ValueError(msg)

edge_tab = {
    '2_3': nm.array([[0,1],
                     [1,2],
                     [2,0]]),
    '2_4': nm.array([[0,1],
                     [1,2],
                     [2,3],
                     [3,0]]),
    '3_4': nm.array([[0,1],
                     [1,2],
                     [2,0],
                     [0,3],
                     [1,3],
                     [2,3]]),
    '3_8': nm.array([[0,1],
                     [1,2],
                     [2,3],
                     [3,0],
                     [4,5],
                     [5,6],
                     [6,7],
                     [7,4],
                     [0,4],
                     [1,5],
                     [2,6],
                     [3,7]]),
}

face_tab = {
    '3_4': nm.array([[0,1,2],
                     [0,1,3],
                     [1,2,3],
                     [0,3,2]]),
    '3_8': nm.array([[0,1,2,3],
                     [0,1,5,4],
                     [1,2,6,5],
                     [2,3,7,6],
                     [3,0,4,7],
                     [4,5,6,7]]),
}

def unique_rows(a):
    a.sort(axis=1)
    order = nm.lexsort(a.T)
    a = a[order]
    diff = nm.diff(a, axis=0)
    ui = nm.ones(len(a), dtype=nm.bool)
    ui[1:] = (diff != 0).any(axis=1)
    uio = nm.zeros_like(ui)
    uio[order] = ui

    sio = nm.ones_like(ui)
    ii = nm.where(nm.invert(ui))
    ui[(ii[0] - 1)] = False
    sio[order] = ui

    return uio, sio

def get_snodes_uedges(conns, etype):

    if etype[0] == '2':
        fci = edge_tab[etype]

    else:
        fci = face_tab[etype]

    nel = conns.shape[0]
    nnd = nm.max(conns) + 1
    nfc = fci.shape[0]
    nnpdfc = fci.shape[1]

    faces = conns[:,fci].reshape((nel * nfc, nnpdfc))
    ufci, sfci = unique_rows(faces)
    ufaces = faces[ufci]
    sfaces = faces[sfci]
    snodes = nm.zeros((nnd,), dtype=nm.bool)
    for ii in sfaces.T:
        snodes[(ii, )] = True

    sndi = nm.where(snodes)

    if etype[0] == '2':
        uedges = ufaces

    else:
        edi = edge_tab[etype]
        ned = edi.shape[0]
        edges = conns[:,edi].reshape((nel * ned, 2))
        uedi, _ = unique_rows(edges)
        uedges = edges[uedi]

    return sndi, uedges



#! /usr/bin/python
# -*- coding: utf-8 -*-

import subprocess
import traceback


import logging
logger = logging.getLogger(__name__)
import datetime


# def write_update_info_in_file():
#     import misc
#     now = datetime.datetime.now() 
def update_by_plan(filename=".update_plan.yaml", update_periode_days=10):
    """
    :return: True or False
    """
    retval = False
    now = datetime.datetime.now() 
    try:
        # this import is necessary here. This module requires scipy and it is 
        # not installed yet
        import misc
        data = misc.obj_from_file(filename)

        string_date = data['update_datetime']
        dt = datetime.datetime.strptime(string_date, "%Y-%m-%d %H:%M:%S.%f")
        dt_delta = datetime.timedelta(days=update_periode_days)
        if now > (dt + dt_delta):
            retval = True
            data['update_datetime'] = str(now)
            misc.obj_to_file(data, filename)

        # data['update_datetime'] = 
    except:
        retval = True
        data = {}
        data['update_datetime'] = str(now)
        misc.obj_to_file(data, filename)

    return retval

def update(dry_run=False):
    if update_by_plan():
        make_update(dry_run)

def make_update(dry_run=False):
    import os.path as op
    path_to_script = op.dirname(op.abspath(__file__))
    path_to_base = op.abspath(op.join(path_to_script, '../'))
    # release_type = 'devel'
    # release_type = 'stable'

    if not dry_run:
        try:
            subprocess.call('git pull', shell=True)
        except:
            logger.warning('Problem with git pull')
    conda_ok = True
    print ('Updating conda root modules')
    try:
        cmd = ["conda", "install", "--yes", "--file",
               op.join(path_to_base, "requirements_conda_root.txt")]
        if dry_run:
            cmd.append('--dry-run')
        subprocess.call(cmd)
    except:
        logger.warning('Problem with conda root update')
        traceback.print_exc()
        conda_ok = False

    print ('Updating conda modules')
    try:
        cmd = ["conda", "install", "--yes",
               # "-c", 'luispedro ',
               '-c', 'SimpleITK', "-c", "menpo", "--file",
               op.join(path_to_base, "requirements_conda.txt")]
        if dry_run:
            cmd.append('--dry-run')
        subprocess.call(cmd)
    except:
        logger.warning('Problem with conda update')
        traceback.print_exc()
        conda_ok = False
        # try:
        #     install_and_import('wget', '--user')

    print ('Updating pip modules')
    try:

        cmd = ["pip", "install", '-U', '--no-deps']
        if not conda_ok:
            cmd.append('--user')
        cmd = cmd + ["-r", op.join(path_to_base, "requirements_pip.txt")]
        if dry_run:
            cmd.insert(1, '-V')
        subprocess.call(cmd)
    except:
        logger.warning('Problem with pip update')
        traceback.print_exc()

    # branch_name = subprocess.check_output(['git', 'branch'])
    # # if we found stabel (find is not -1), we should use specific version
    # if branch_name.find('* stable') != -1 or release_type == 'stable':
    #     print ('Stable version prerequisities')
    #     use_specifed_version = 1
    # else:
    #     use_specifed_version = 0

    # update submodules codes
    # This can update only one package to actual or defined version
    # installed version can be checked with
    #     pip show sed3
    # pipshell = 'pip install -U --no-deps --user '
    # pppysegbase = ['pysegbase', 'pysegbase==1.0.13']
    # ppio3d = ['io3d', 'io3d==1.0.5']
    # ppsed3 = ['sed3', 'sed3==1.0.4']
    # if not dry_run:
    #     try:
    #         subprocess.call('git pull', shell=True)
    #         subprocess.call(
    #             'git submodule update --init --recursive', shell=True)
    #         subprocess.call(
    #             pipshell + pppysegbase[use_specifed_version], shell=True)
    #         subprocess.call(pipshell + ppio3d[use_specifed_version],
    #             shell=True)
    #         subprocess.call(pipshell + ppsed3[use_specifed_version],
    #             shell=True)
    #         # skelet3d is not in pipy
    #         # subprocess.call(
    #         #      'pip install -U --no-deps skelet3d --user', shell=True)
    #     except:
    #         print ('Probem with git submodules')
    #         print (traceback.format_exc())


def install_and_import(package, pip_params=[]):
    import importlib
    try:
        importlib.import_module(package)
    except ImportError:
        import pip
        main_args = ['install'] + pip_params
        pip.main(main_args)
    finally:
        globals()[package] = importlib.import_module(package)


def main():
    logger = logging.getLogger()

    logger.setLevel(logging.WARNING)
    ch = logging.StreamHandler()
    logger.addHandler(ch)
    update()

if __name__ == "__main__":
    main()



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright © %YEAR% %USER% <%MAIL%>
#
# Distributed under terms of the %LICENSE% license.

"""
%HERE%
"""

import logging

logger = logging.getLogger(__name__)
import argparse

from PyQt4 import QtGui
# from mainwindow import Ui_MainWindow

class Login(QtGui.QDialog):
    def __init__(self, parent=None, checkLoginFcn=None):
        super(Login, self).__init__(parent)
        self.textName = QtGui.QLineEdit(self)
        self.textPass = QtGui.QLineEdit(self)
        self.textPass.setEchoMode(QtGui.QLineEdit.Password)
        self.buttonLogin = QtGui.QPushButton('Login', self)
        self.buttonLogin.clicked.connect(self.handleLogin)
        layout = QtGui.QVBoxLayout(self)
        layout.addWidget(self.textName)
        layout.addWidget(self.textPass)
        layout.addWidget(self.buttonLogin)
        if checkLoginFcn is None:
            self.checkLoginFcn = self.checkLogin
        else:
            self.checkLoginFcn = checkLoginFcn

    def checkLogin(self, text_name, text_pass):
        if (text_name == 'foo' and
                    text_pass == 'bar'):
            return True
        else:
            return False

    def handleLogin(self):
        if self.checkLoginFcn(str(self.textName.text()), str(self.textPass.text())):
            self.accept()
        else:
            QtGui.QMessageBox.warning(
                self, 'Error', 'Bad user or password')

class Window(QtGui.QMainWindow):
    def __init__(self, parent=None):
        super(Window, self).__init__(parent)
        # self.ui = Ui_MainWindow()
        # self.ui.setupUi(self)



def main():
    logger = logging.getLogger()

    logger.setLevel(logging.DEBUG)
    ch = logging.StreamHandler()
    logger.addHandler(ch)

    # create file handler which logs even debug messages
    # fh = logging.FileHandler('log.txt')
    # fh.setLevel(logging.DEBUG)
    # formatter = logging.Formatter(
    #     '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    # fh.setFormatter(formatter)
    # logger.addHandler(fh)
    # logger.debug('start')

    # input parser
    parser = argparse.ArgumentParser(
        description=__doc__
    )
    parser.add_argument(
        '-i', '--inputfile',
        default=None,
        # required=True,
        help='input file'
    )
    parser.add_argument(
        '-d', '--debug', action='store_true',
        help='Debug mode')
    args = parser.parse_args()

    if args.debug:
        ch.setLevel(logging.DEBUG)

    import sys
    app = QtGui.QApplication(sys.argv)
    login = Login()

    if login.exec_() == QtGui.QDialog.Accepted:
        window = Window()
        window.show()
        sys.exit(app.exec_())

if __name__ == "__main__":
    main()


# ! /usr/bin/python
# -*- coding: utf-8 -*-
"""
Experiment s \"barevným\" modelem jater.

Pro spuštění zkuste help:

python experiments/20130919_liver_statistics.py --help

Měřené vlastnosti se přidávají do get_features().
Pro přidání dalších dat, editujte příslušný yaml soubor.

"""

# import funkcí z jiného adresáře
import sys
import os.path

path_to_script = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(path_to_script, "../lisa/"))
sys.path.append(os.path.join(path_to_script, "../extern/pyseg_base/src"))
sys.path.append(os.path.join(path_to_script,
                             "../extern/sed3/"))
sys.path.append(os.path.join(path_to_script,
                             "../extern/lbp/"))
# sys.path.append(os.path.join(path_to_script, "../extern/"))
# import featurevector

import logging
logger = logging.getLogger(__name__)

import json
# import apdb
#  apdb.set_trace();
# import scipy.io
import numpy as np
import scipy
# from scipy import sparse
# import traceback
import itertools

# ----------------- my scripts --------
import sed3
# import dcmreaddata1 as dcmr
# import dcmreaddata as dcmr
import argparse
# import sed3

import misc
import qmisc
from io3d import datareader
import matplotlib.pyplot as plt
from lisa import experiments
import texture_features as tfeat


def feat_hist_by_segmentation(data3d_orig, data3d_seg, voxelsize_mm=[1],
                              visualization=True):
    bins = range(-1024, 1024, 1)
    bins = range(-512, 512, 1)
    hist1, bin_edges1 = np.histogram(data3d_orig[data3d_seg > 0], bins=bins)
    hist2, bin_edges2 = np.histogram(data3d_orig[data3d_seg <= 0], bins=bins)
    # import pdb; pdb.set_trace()
    if visualization:
        plt_liver = plt.step(bin_edges1[1:], hist1)
        plt_rest = plt.step(bin_edges2[1:], hist2)
        plt.legend([plt_liver, plt_rest], ['Liver', 'Other tissue'])
        # plt.plot(bin_edges1[1:], hist1, bin_edges2[1:], hist2)
        plt.show()

    vvolume = np.prod(voxelsize_mm)
    fv_hist = {'hist1': hist1 * vvolume,
               'hist2': hist2 * vvolume,
               'bins': bins
               }
    return fv_hist


def feat_hist(data3d_orig):
    # bins = range(-1024, 1024, 1)
    # bins = range(-512, 512, 1)
    # bins = range(-512, 512, 10)
    # bins = range(-512, 512, 64)
    bins = range(-512, 512, 100)
    hist1, bin_edges1 = np.histogram(data3d_orig, bins=bins)
    return hist1


def super_feat_hist(data3d_orig, parameter):
    return feat_hist(data3d_orig)
    # print parameter


def f_greycomatrix(data3d):
    import skimage
    skimage.feature.greycomatrix(data3d[:, :, 0], [5], )


def lbp(data3d_orig, data3d_seg, visualization=True):
    import realtime_lbp as real_lib
    realLbp = real_lib.loadRealtimeLbpLibrary()
    lbpRef = np.zeros([1, 256])
    lbpRef = real_lib.realTimeLbpImNp(realLbp, data3d_orig[:, :, 1])
    return lbpRef


def lbp3d(data3d_orig, filename, visualization=True):
    from lbp import lbp3d
    lib3d = lbp3d.load()
    f = open(filename, 'r')
    maskJSON = json.load(f)
    mask = maskJSON['mask']
    lbp3d.coordToPoints(
        mask,
        data3d_orig.shape[2],
        data3d_orig.shape[1])
    res = lbp3d.compute(
        lib3d,
        data3d_orig,
        mask)
    return res


def f_lbp3d(data3d_orig):
    return lbp3d(
        data3d_orig,
        '/home/petr/Dokumenty/git/lbpLibrary/masks/mask3D_8_4.json',
        True)


def get_features(data3d_orig, data3d_seg, feature_fcn, feature_fcn_params=[],
                 visualization=True):
    u"""
    Sem doplníme všechny naše měření.

    Pro ukázku jsem vytvořil měření
    histogramu.
    data3d_orig: CT data3d_orig
    data3d_seg: jedničky tam, kde jsou játra
    feature_fcn_plus_params: list of size 2: [feature_function, [3, 'real']]

    There are two possible feature_fcn formats
     * method
     * object with method 'features' and attribute 'description'
    """
    # feature_fcn, feature_fcn_params = feature_fcn_plus_params

    if hasattr(feature_fcn, 'description'):
        feature_fcn = feature_fcn.features

    featur = feature_fcn(data3d_orig, *feature_fcn_params)
    # featur = {}
    # featur['hist'] = feat_hist(data3d_orig, visualization)
    # featur['lbp'] = lbp(data3d_orig, data3d_seg, visualization)

    return featur


def get_features_in_tiles(
        data3d_orig,
        data3d_seg,
        tile_shape,
        feature_fcn,
        feature_fcn_params):
    """
    Computes features for small blocks of image data (tiles).

    cindexes: indexes of tiles
    features_t: numpy array of features
    segmentation_cover: np array with coverages of tiles with segmentation
    (float 0 by 1)

    """
    cindexes = cutter_indexes(data3d_orig.shape, tile_shape)
# create empty list of defined length
    features_t = [None] * len(cindexes)
    seg_cover_t = [None] * len(cindexes)
    logger.debug(
        " ##    get fv" + str(len(cindexes)) + " dsh " +
        str(data3d_orig.shape) + 'tile_shape ' + str(tile_shape))
    for i in range(0, len(cindexes)):
        cindex = cindexes[i]
        tile_orig = experiments.getArea(data3d_orig, cindex, tile_shape)
        tile_seg = experiments.getArea(data3d_seg, cindex, tile_shape)
        tf = get_features(tile_orig, tile_seg, feature_fcn, feature_fcn_params,
                          visualization=False)
        sc = np.sum(tile_seg > 0).astype(np.float) / np.prod(tile_shape)
        features_t[i] = tf
        seg_cover_t[i] = sc
    return cindexes, features_t, seg_cover_t
    # if (tile_seg == 1).all():
    #    pass


def cutter_indexes(shape, tile_shape):
    """
    Make indexes for cutting.

    shape: shape of cutted data
    tile_shape: shape of tile

    """
    # TODO přepis r1?
    # r0 = range(0, shape[0] - tile_shape[0] + 1, tile_shape[0])
    # r1 = range(1, shape[1] - tile_shape[1] + 1, tile_shape[1])
    # r2 = range(2, shape[2] - tile_shape[2] + 1, tile_shape[2])
    r0 = range(0, shape[0], tile_shape[0])
    r1 = range(0, shape[1], tile_shape[1])
    r2 = range(0, shape[2], tile_shape[2])
    cut_iterator = itertools.product(r0[:-1], r1[:-1], r2[:-1])
    return list(cut_iterator)


# @TODO dodělat rozsekávač
def cut_tile(data3d, cindex, tile_shape):
    """ Function is similar to experiments.getArea(). """

    upper_corner = cindex + np.array(tile_shape)
    # print cindex, "    tile shape ", tile_shape, ' uc ', upper_corner,\
    #    ' dsh ', data3d.shape

    return data3d[cindex[0]:upper_corner[0],
                  cindex[1]:upper_corner[1],
                  cindex[2]:upper_corner[2]
                  ]


def arrange_to_tiled_data(cindexes, tile_shape, data3d_shape, labels_lin):
    """ Creates 3D image with values of labels.  """

    labels = np.zeros(data3d_shape, dtype=type(labels_lin[0]))
    for i in range(0, len(cindexes)):
        cindex = cindexes[i]

# TODO labels shape
        labels = experiments.setArea(labels, cindex, tile_shape, labels_lin[i])
    return labels


def generate_input_yaml_metadata():
    pass


def sample_input_data():
    inputdata = {
        'basedir': '/home/mjirik/data/medical/',
        'data': [
            {'sliverseg': 'data_orig/sliver07/training-part1/liver-seg001.mhd', 'sliverorig': 'data_orig/sliver07/training-part1/liver-orig001.mhd'},  # noqa
            {'sliverseg': 'data_orig/sliver07/training-part1/liver-seg002.mhd', 'sliverorig': 'data_orig/sliver07/training-part1/liver-orig002.mhd'},  # noqa
        ]
    }

    sample_data_file = os.path.join(path_to_script,
                                    "20130919_liver_statistics.yaml")
    # print sample_data_file, path_to_script
    misc.obj_to_file(inputdata, sample_data_file, filetype='yaml')

# def voe_metric(vol1, vol2, voxelsize_mm):


def write_csv(data, filename="20130919_liver_statistics.yaml"):
    import csv
    with open(filename, 'wb') as csvfile:
        spamwriter = csv.writer(csvfile,
                                delimiter=';',
                                quotechar='"',
                                quoting=csv.QUOTE_MINIMAL
                                )
        for label in data:
            spamwriter.writerow([label] + data[label])
            # spamwriter.writerow(['Spam', 'Lovely Spam', 'Wonderful Spam'])


# def training_dataset_prepare
def read_data_orig_and_seg(inputdata, i):
    """ Loads data_orig and data_seg from yaml file
    """

    reader = datareader.DataReader()
    data3d_a_path = os.path.join(inputdata['basedir'],
                                 inputdata['data'][i]['sliverseg'])
    data3d_a, metadata_a = reader.Get3DData(data3d_a_path)

    data3d_b_path = os.path.join(inputdata['basedir'],
                                 inputdata['data'][i]['sliverorig'])
    data3d_b, metadata_b = reader.Get3DData(data3d_b_path)

    # import pdb; pdb.set_trace()
    data3d_seg = (data3d_a > 0).astype(np.int8)
    data3d_orig = data3d_b

    return data3d_orig, data3d_seg, metadata_a['voxelsize_mm']


def data_preprocessing(data3d, voxelsize_mm, working_voxelsize_mm):
    # @TODO unused delete it
    # working_voxelsize_mm = np.array([2.0, 1.0, 1.0])
    zoom = voxelsize_mm / (1.0 * working_voxelsize_mm)

    data3d_res = scipy.ndimage.zoom(
        data3d,
        zoom,
        mode='nearest',
        order=1
    ).astype(np.int16)
    return data3d_res, working_voxelsize_mm


def data_postprocessing(segmentation_res, voxelsize_mm, working_voxelsize_mm):
    # @TODO unused delete it
    zoom = voxelsize_mm / (1.0 * working_voxelsize_mm)
    segm_orig_scale = scipy.ndimage.zoom(
        segmentation_res,
        1.0 / zoom,
        mode='nearest',
        order=0
    ).astype(np.int8)
    return segm_orig_scale


def save_labels(
        inputfile,
        data3d,
        segmentation,
        feature_fcn,
        feature_fcn_params,
        classif_inst,
        classif_fcn_plus_params,
        voxelsize,
        tile_shape,
        use_voxelsize_norm
        ):
    """
    classif_fcn_plus_params: used for directory name
    """
    path_directory = 'lisa_data/'
    subdirectory = 'experiments/'
    # actual = os.getcwd()
    path_directory = os.path.join(os.path.expanduser('~'), path_directory)
    # Ukládání výsledku do souboru
    if(os.path.exists(path_directory) is False):
        os.makedirs(path_directory)
    path_subdirectory = os.path.join(path_directory, subdirectory)
    if(os.path.exists(path_subdirectory) is False):
        os.makedirs(path_subdirectory)
    # TODO : Main Saving Loop ...
    dataplus = []
    slab = {}
    slab['liver'] = 1
    slab['none'] = 0

# there are two possible ways for fature_fcn. One is function, other is object
# with "description" attribute and function 'features'
    if hasattr(feature_fcn, 'description'):
        feature_fcn_description = feature_fcn.description
    else:
        feature_fcn_description = str(feature_fcn)

    dataplus = {
        # 'segmentation': segmentation[:10, :10, :10].astype(np.int8),
        # 'data3d': data3d[:10, :10, :10].astype(np.int16),
        'segmentation': segmentation.astype(np.int8),
        'data3d': data3d.astype(np.int16),
        'processing_information': {
            'feature_fcn': feature_fcn_description,
            'feature_fcn_params': str(feature_fcn_params),
            'classif_fcn_name': str(classif_inst.__class__.__name__),
            'classif_fcn': str(classif_inst)
        },
        'voxelsize_mm': voxelsize,
        'slab': slab}
    # inputfilename = path_leaf(inputfile)
    str_feat_params = __struct_to_string_for_filename(feature_fcn_params)
    str_clf_params = __struct_to_string_for_filename(str(classif_inst))
    str_clf_params = __struct_to_string_for_filename(
        str(classif_fcn_plus_params))

    vsnorm_string = "vsnF_"
    if use_voxelsize_norm:
        vsnorm_string = "vsnT_"

# construct filename
    experiment_dirname =\
        feature_fcn_description + '_' + \
        str_feat_params + '_' +\
        vsnorm_string +\
        classif_inst.__class__.__name__ + '_' + \
        str_clf_params + '_' +\
        str(tile_shape[0]) + '_' + str(tile_shape[1]) + '_' +\
        str(tile_shape[2])

    filename = __struct_to_string_for_filename(inputfile + '.pklz')

    path_to_file = os.path.join(path_subdirectory,
                                experiment_dirname, filename)
    misc.obj_to_file(dataplus, path_to_file, filetype='pklz')
    # os.chdir(actual)


# save info to dir
    info = {
        'feature_fcn': feature_fcn_description,
        'feature_fcn_params': str(feature_fcn_params),
        'classif_fcn': str(classif_inst)
        }
    infofilepath = os.path.join(path_subdirectory,
                                experiment_dirname, 'info.yaml')
    misc.obj_to_file(info, infofilepath, 'yaml')


def norm_voxelsize(data3d_orig, data3d_seg, voxelsize_mm,
                   working_voxelsize_mm):

    data3d_orig_res = qmisc.resize_to_mm(data3d_orig,
                                         voxelsize_mm, working_voxelsize_mm)
    data3d_seg_res = qmisc.resize_to_mm(data3d_seg,
                                        voxelsize_mm, working_voxelsize_mm)

    return data3d_orig_res, data3d_seg_res


def one_exp_set_training(inputdata, tile_shape,
                         feature_fcn_plus_params,
                         classif_fcn_plus_params,
                         use_voxelsize_norm=False,
                         working_voxelsize_mm=[1, 1, 1],
                         visualization=False):
    """
    Training of experiment.
    """
    features_t_all = []
    labels_train_lin_all = []
    indata_len = len(inputdata['data'])
    features_t_all = []
    # indata_len = 3
    classif_fcn, classif_fcn_params = classif_fcn_plus_params
    logger.debug('number of data files ' + str(indata_len))

    for i in range(0, indata_len):
        data3d_orig, data3d_seg, voxelsize_mm = read_data_orig_and_seg(
            inputdata, i)

        if use_voxelsize_norm:
            data3d_orig, data3d_seg = norm_voxelsize(
                data3d_orig, data3d_seg,
                voxelsize_mm, working_voxelsize_mm)

        if visualization:
            pyed = sed3.sed3(data3d_orig,
                                                 contour=data3d_seg)
            pyed.show()
        logger.debug('data shape ' + str(data3d_orig.shape))
        fv_t = get_features_in_tiles(data3d_orig, data3d_seg, tile_shape,
                                     feature_fcn_plus_params[0],
                                     feature_fcn_plus_params[1])
        cidxs, features_t, seg_cover_t = fv_t
        labels_train_lin_float = np.array(seg_cover_t)
        labels_train_lin = (
            labels_train_lin_float > 0.5).astype(np.int8).tolist()

        features_t_all = features_t_all + features_t
        labels_train_lin_all = labels_train_lin_all + labels_train_lin

    # if there are named variables use dict unpacking
    if isinstance(classif_fcn_params, dict):
        clf = classif_fcn(**classif_fcn_params)
    else:
        clf = classif_fcn(*classif_fcn_params)

    clf.fit(features_t_all, labels_train_lin_all)
    # import ipdb; ipdb.set_trace()  # noqa BREAKPOINT
    return clf


def __struct_to_string_for_filename(params):
    sarg = str(params)
    sarg = sarg.replace('{', '')
    sarg = sarg.replace('}', '')
    sarg = sarg.replace('[', '')
    sarg = sarg.replace(']', '')
    sarg = sarg.replace('<', '')
    sarg = sarg.replace('>', '')
    sarg = sarg.replace('"', '')
    sarg = sarg.replace("'", '')
    sarg = sarg.replace(" ", '')
    sarg = sarg.replace(",", '_')
    sarg = sarg.replace(":", '-')
    sarg = sarg.replace("\\", '-')
    sarg = sarg.replace("/", '-')
    if len(sarg) > 100:
        sarg = sarg[:100]
    return sarg


def one_exp_set_testing(
        inputdata, tile_shape,
        feature_fcn_plus_params,
        clf,
        classif_fcn_plus_params,
        use_voxelsize_norm=False,
        working_voxelsize_mm=[1, 1, 1],
        visualization=False):
    indata_len = len(inputdata['data'])
    # indata_len = 3
    feature_fcn, feature_fcn_params = feature_fcn_plus_params

    for i in range(0, indata_len):
        data3d_orig_orig, data3d_seg_orig, voxelsize_mm = \
            read_data_orig_and_seg(inputdata, i)
        data3d_orig = data3d_orig_orig
        data3d_seg = data3d_seg_orig

        if use_voxelsize_norm:
            data3d_orig, data3d_seg = norm_voxelsize(
                data3d_orig_orig, data3d_seg_orig,
                voxelsize_mm, working_voxelsize_mm)

        if visualization:
            pyed = sed3.sed3(data3d_orig,
                                                 contour=data3d_seg)
            pyed.show()

        fv_t = get_features_in_tiles(data3d_orig, data3d_seg, tile_shape,
                                     feature_fcn,
                                     feature_fcn_params)
        cidxs, features_t, seg_cover_t = fv_t
        # we are ignoring seg_cover_t which gives us information about
        # segmentation

        labels_lin = clf.predict(features_t)

        d_shp = data3d_orig.shape

        segmentation = arrange_to_tiled_data(cidxs, tile_shape, d_shp,
                                             labels_lin)

        if use_voxelsize_norm:
            segmentation = qmisc.resize_to_shape(
                segmentation,
                data3d_orig_orig.shape)
# @TODO změnil jsem to. Už zde není ukazatel na klasifikátor, ale přímo
# natrénovaný klasifikátor.
        save_labels(
            inputdata['data'][i]['sliverorig'], data3d_orig_orig, segmentation,
            feature_fcn, feature_fcn_params, clf,
            classif_fcn_plus_params, voxelsize_mm, tile_shape,
            use_voxelsize_norm)
        # ltl = (labels_train_lin_float * 10).astype(np.int8)
        # labels_train = arrange_to_tiled_data(cidxs, tile_shape,
        #                                     d_shp, ltl)

        # pyed = sed3.sed3(labels_train, contour=labels)
        if visualization:
            pyed = sed3.sed3(
                data3d_seg,
                contour=segmentation)
            pyed.show()
    pass


def one_exp_set_for_whole_dataset(
        training_yaml,
        testing_yaml,
        tile_shape,
        feature_fcn_plus_params,
        classif_fcn_plus_params,
        train,
        use_voxelsize_norm,
        working_voxelsize_mm,
        visualization=False):
    fvall = []
    # fv_tiles = []
    print "classif_fcn ", classif_fcn_plus_params
    print "feature_fcn ", feature_fcn_plus_params
    clf = one_exp_set_training(
        training_yaml, tile_shape,
        feature_fcn_plus_params,
        classif_fcn_plus_params,
        use_voxelsize_norm,
        working_voxelsize_mm,
        visualization=False)

    logger.info('run testing')
    one_exp_set_testing(testing_yaml, tile_shape,
                        feature_fcn_plus_params, clf,
                        classif_fcn_plus_params,
                        use_voxelsize_norm,
                        working_voxelsize_mm,
                        visualization=visualization)

# @TODO vracet něco inteligentního, fvall je prázdný
    return fvall


def make_product_list(list_of_feature_fcn, list_of_classifiers):
    #   TODO work with list_of_feature_fcn and list_of_classifiers
    featrs_plus_classifs = itertools.product(list_of_feature_fcn,
                                             list_of_classifiers)
    return featrs_plus_classifs


def experiment(training_yaml_path, testing_yaml_path,  featers_plus_classifs,
               tile_shape,
               use_voxelsize_norm,
               working_voxelsize_mm,
               visualization=False, train=False):

    training_yaml = misc.obj_from_file(training_yaml_path, filetype='yaml')
    testing_yaml = misc.obj_from_file(testing_yaml_path, filetype='yaml')

    # import ipdb; ipdb.set_trace()  # noqa BREAKPOINT

    results = []

    for fpc in featers_plus_classifs:
        feature_fcn = fpc[0]
        classif_fcn = fpc[1]

        fvall = one_exp_set_for_whole_dataset(
            training_yaml, testing_yaml, tile_shape,
            feature_fcn, classif_fcn,
            train,
            use_voxelsize_norm,
            working_voxelsize_mm,
            visualization)

        result = {'params': str(fpc), 'fvall': fvall}
        results.append(result)
        print results

    return results


def prepared_classifiers_by_string(names):
    from sklearn import svm
    from sklearn.naive_bayes import GaussianNB  # noqa
    from sklearn.mixture import GMM  # noqa
    from sklearn import tree  # noqa
    from sklearn.ensemble import RandomForestClassifier, AdaBoostClassifier  # noqa
    from sklearn.lda import LDA  # noqa
    from sklearn.qda import QDA  # noqa
    import classification
    dict_of_classifiers = {
        'GaussianNB': [GaussianNB, []],
        'SVC': [svm.SVC, []],
        'DT': [tree.DecisionTreeClassifier, []],
        'GMM2': [classification.GMMClassifier,
                 {'n_components': 2, 'covariance_type': 'full'}],
        'SVClin': [svm.SVC, {'kernel': 'linear'}],
        'SVCrbf': [svm.SVC, {'kernel': 'rbf'}],
        'SVCpoly': [svm.SVC, {'kernel': 'poly'}],
        'RF': [RandomForestClassifier, []],
        'LDA': [LDA, []],
        'QDA': [QDA, []],
        }

    selected_classifiers = [dict_of_classifiers[name] for name in names]
    return selected_classifiers


def prepared_texture_features_by_string(names):
    """
    There are two possible feature_fcn formats
     * method
     * object with method 'features' and attribute 'description'
    """
    gf = tfeat.GaborFeatures()  # noqa
    glcmf = tfeat.GlcmFeatures()  # noqa
    haralick = tfeat.HaralickFeatures()  # noqa
    hist_gf = tfeat.FeaturesCombinedFeatures(feat_hist, gf.feats_gabor)
    hist_glcm = tfeat.FeaturesCombinedFeatures(feat_hist, glcmf.feats_glcm)
    hist_glcm_gf = tfeat.FeaturesCombinedFeatures(
        feat_hist,
        glcmf.feats_glcm,
        gf.feats_gabor
    )

    dict_of_feature_fcn = {
        'hist': [feat_hist, []],
        'gf': [gf.feats_gabor, []],
        'glcm': [glcmf.feats_glcm, []],
        'haralick': [haralick.feats_haralick, [True]],
        'hist_gf': [hist_gf, []],
        'hist_glcm': [hist_glcm, []],
        'hist_glcm_gf': [hist_glcm_gf, []],
    }

    selected_classifiers = [dict_of_feature_fcn[name] for name in names]
    return selected_classifiers


def main():

    # logger = logging.getLogger(__name__)
    logger = logging.getLogger()

    logger.setLevel(logging.WARNING)
    ch = logging.StreamHandler()
    logger.addHandler(ch)

    # logger.debug('input params')

    parser = argparse.ArgumentParser(
        description='Compute features on liver and other tissue.')
    parser.add_argument('-tr', '--training_yaml_path',
                        help='Input yaml file.' +
                        " You can check sample with -si parameter.",
                        default="20130919_liver_statistics.yaml")
    parser.add_argument('-te', '--testing_yaml_path', help='Input yaml file.' +
                        " You can check sample with -si parameter.",
                        default=None)
    parser.add_argument('-si', '--sampleInput', action='store_true',
                        help='generate sample intput data', default=False)
    parser.add_argument('-v', '--visualization',  action='store_true',
                        help='Turn on visualization', default=False)
    parser.add_argument('-fc', '--features_classifs',  action='store_true',
                        help='Read features and classifs list from file',
                        default=False)
    parser.add_argument('-o', '--output', help='output file',
                        default="20130919_liver_statistics_results.pkl")
    parser.add_argument('-t', '--train', help='Training', default=False,
                        action='store_true'
                        )
    parser.add_argument(
        '-cl', '--classifers',
        help='classifer by string: "SVC", or "GaussianNB", ...',
        nargs='+', type=str, default=['SVC']
    )
    parser.add_argument(
        '-fe', '--features',
        help='features by string: "hist", or "glcm", ...',
        nargs='+', type=str, default=['hist']
    )
    args = parser.parse_args()

    if args.sampleInput:
        sample_input_data()
    # input parser
    # path_to_yaml = os.path.join(path_to_script, args.input)
    # training_yaml_path = args.training_yaml_path
    # testing_yaml_path = args.testing_yaml_path
    if args.testing_yaml_path is None:
        print 'testing is same as training'
        args.testing_yaml_path = args.training_yaml_path

    # write_csv(fvall)
    gf = tfeat.GaborFeatures()  # noqa
    glcmf = tfeat.GlcmFeatures()  # noqa
    haralick = tfeat.HaralickFeatures()  # noqa

    list_of_feature_fcn = prepared_texture_features_by_string(args.features)

    list_of_classifiers = prepared_classifiers_by_string(args.classifers)
    tile_shape = [10, 50, 50]

    if args.features_classifs:
        import features_classifs
        featrs_plus_classifs = features_classifs.fc
    else:
        featrs_plus_classifs = make_product_list(list_of_feature_fcn,
                                                 list_of_classifiers)

    result = experiment(args.training_yaml_path, args.testing_yaml_path,
                        featrs_plus_classifs, tile_shape=tile_shape,
                        use_voxelsize_norm=True,
                        working_voxelsize_mm=[1, 1, 1],
                        visualization=args.visualization, train=args.train)

# Ukládání výsledku do souboru
    output_file = os.path.join(path_to_script, args.output)
    misc.obj_to_file(result, output_file, filetype='pickle')

if __name__ == "__main__":
    main()



#! /opt/local/bin/python
# -*- coding: utf-8 -*-
"""
Show dicom data with overlay
"""

# import funkcí z jiného adresáře
import sys
import os.path

path_to_script = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(path_to_script, "../extern/pyseg_base/src"))
sys.path.append(os.path.join(path_to_script,
                             "../extern/sed3/"))
#sys.path.append(os.path.join(path_to_script, "../extern/"))
#import featurevector

import logging
logger = logging.getLogger(__name__)


#import apdb
#  apdb.set_trace();
#import scipy.io
import numpy as np
#import scipy
#from scipy import sparse
#import traceback
#import time
#import audiosupport

# ----------------- my scripts --------
import argparse
import sed3

#import segmentation
import qmisc
import misc
from io3d import datareader
#import config
#import numbers


def saveOverlayToDicomCopy(input_dcmfilelist, output_dicom_dir, overlays,
                           crinfo, orig_shape):
    """ Save overlay to dicom. """
    import datawriter as dwriter

    if not os.path.exists(output_dicom_dir):
        os.mkdir(output_dicom_dir)

    # uncrop all overlays
    for key in overlays:
        overlays[key] = qmisc.uncrop(overlays[key], crinfo, orig_shape)

    dw = dwriter.DataWriter()
    dw.DataCopyWithOverlay(input_dcmfilelist, output_dicom_dir, overlays)


def readData3d(dcmdir):
    qt_app = None
    if dcmdir is None:
        from PyQt4 import QtGui
#QApplication
        qt_app = QtGui.QApplication(sys.argv)
# same as  data_reader_get_dcmdir_qt
#        from PyQt4.QtGui import QFileDialog, QApplication
#        dcmdir = QFileDialog.getExistingDirectory(
#                caption='Select DICOM Folder',
#                options=QFileDialog.ShowDirsOnly)
#        dcmdir = "%s" %(dcmdir)
#        dcmdir = dcmdir.encode("utf8")
        dcmdir = datareader.get_dcmdir_qt(qt_app)
    reader = datareader.DataReader()
    data3d, metadata = reader.Get3DData(dcmdir, qt_app=None)

    return data3d, metadata, qt_app


def save_config(cfg, filename):
    cfg
    misc.obj_to_file(cfg, filename, filetype="yaml")


def main():

    #logger = logging.getLogger(__name__)
    logger = logging.getLogger()

    logger.setLevel(logging.WARNING)
    ch = logging.StreamHandler()
    logger.addHandler(ch)

    #logger.debug('input params')

## read confguraton from file, use default values from OrganSegmentation

    # input parser
    parser = argparse.ArgumentParser(
        description=' Show dicom data with overlay')
    parser.add_argument('-i', '--inputdatapath',
                        default='',
                        help='path to data dir')
    args_obj = parser.parse_args()
    args = vars(args_obj)
    #print args["arg"]

    reader = datareader.DataReader()
    data3d, metadata = reader.Get3DData(args['inputdatapath'], qt_app=None)
    overlays = reader.GetOverlay()
    overlay = np.zeros(data3d.shape, dtype=np.int8)
    print "overlays ", overlays.keys()
    for key in overlays:
        overlay += overlays[key]

    #import ipdb; ipdb.set_trace() # BREAKPOINT
    pyed = sed3.sed3(data3d, contour=overlay)
    pyed.show()

    #print savestring
#    import pdb; pdb.set_trace()
    return

if __name__ == "__main__":
    main()



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright © 2015 mjirik <mjirik@hp-mjirik>
#
# Distributed under terms of the MIT license.

"""
Crop, resize, reshape
"""

import logging
logger = logging.getLogger(__name__)
import argparse
import scipy
import numpy as np

import qmisc

def select_labels(segmentation, labels):
    ds = np.zeros(segmentation.shape, np.bool)
    for lab in labels:
        print "print zpracovavam ", lab
        dadd = (segmentation == lab)

        ds = ds | dadd

    return ds

def unbiased_brick_filter(binary_data, crinfo):
    """
    return only binary object which suits with unbiased brick 

    :param data: 3D ndimage data
    :param crinfo: crinfo 
    http://www.stereology.info/the-optical-disector-and-the-unbiased-brick/
    """
    binary_data = binary_data.copy()
    crinfo = qmisc.fix_crinfo(crinfo)

    imlab, num_features = scipy.ndimage.measurements.label(binary_data)

    brick_neg_mask = np.zeros(binary_data.shape, dtype=np.uint8)
    brick_neg_mask[
            crinfo[0][0]:crinfo[0][1],
            crinfo[1][0]:crinfo[1][1],
            crinfo[2][0]:crinfo[2][1]
        ] = 1

    exclude_mask = np.zeros(binary_data.shape, dtype=np.uint8)
    exclude_mask[:,:, crinfo[2][1]:] = 1
    exclude_mask[:,crinfo[1][1]:, crinfo[2][0]:] = 1
    exclude_mask[:crinfo[0][0],crinfo[1][0]:, crinfo[2][0]:] = 1

    # remove what is not in touch with brick
    imlab = keep_what_is_in_touch_with_mask(
            imlab, brick_neg_mask, max_label=num_features)
    # remove what is in touch with exclude
    imlab = remove_what_is_in_touch_with_mask(imlab, exclude_mask)




    return (imlab > 0).astype(binary_data.dtype)
    


    # return data[
    #     __int_or_none(crinfo[0][0]):__int_or_none(crinfo[0][1]),
    #     __int_or_none(crinfo[1][0]):__int_or_none(crinfo[1][1]),
    #     __int_or_none(crinfo[2][0]):__int_or_none(crinfo[2][1])
    #     ]

def keep_what_is_in_touch_with_mask(imlab, keep_mask, max_label):
    datatmp = imlab * keep_mask
    nz = np.nonzero(datatmp)
    labels_to_keep = imlab[nz[0], nz[1], nz[2]]
    labels_to_keep = np.unique(labels_to_keep)

    for lab in range(0, max_label):
        if lab in labels_to_keep:
            pass
        else:
            imlab[imlab == lab] = 0

    return imlab

def remove_what_is_in_touch_with_mask(imlab, exclude_mask):
    datatmp = imlab * exclude_mask
    nz = np.nonzero(datatmp)
    labels_to_exclude = imlab[nz[0], nz[1], nz[2]]
    labels_to_exclude = np.unique(labels_to_exclude)


    for lab in labels_to_exclude:
        imlab[imlab == lab] = 0

    return imlab



def add_seeds_mm(data_seeds, voxelsize_mm, z_mm, x_mm, y_mm, label, radius, width=1):

    """
    Function add circle seeds to one slice with defined radius.

    It is possible set more seeds on one slice with one dimension

    x_mm, y_mm coordinates of circle in mm. It may be array.
    z_mm = slice coordinates  in mm. It may be array
    :param label: one number. 1 is object seed, 2 is background seed
    :param radius: is radius of circle in mm
    :param width: makes circle with defined width (repeat circle every milimeter)

    """

    # this do not work for ndarrays
    # if type(x_mm) is not list:
    #     x_mm = [x_mm]
    # if type(y_mm) is not list:
    #     x_mm = [y_mm]
    # if type(z_mm) is not list:
    #     z_mm = [z_mm]
    z_mm = np.asarray(z_mm)
    # repeat circle every milimiter
    for i in range(0, width + 1):
        data_seeds = _add_seeds_mm_in_one_slice(data_seeds, voxelsize_mm, z_mm + i, x_mm, y_mm, label, radius)
    return data_seeds

def _add_seeds_mm_in_one_slice(data_seeds, voxelsize_mm, z_mm, x_mm, y_mm, label, radius):
    x_mm = np.asarray(x_mm)
    y_mm = np.asarray(y_mm)
    z_mm = np.asarray(z_mm)

    for i in range(0, len(x_mm)):

        # xx and yy are 200x200 tables containing the x and y coordinates
        # values. mgrid is a mesh creation helper
        xx, yy = np.mgrid[
                 :data_seeds.shape[1],
                 :data_seeds.shape[2]
                 ]
        # circles contains the squared distance to the (100, 100) point
        # we are just using the circle equation learnt at school
        circle = (
                     (xx - x_mm[i] / voxelsize_mm[1]) ** 2 +
                     (yy - y_mm[i] / voxelsize_mm[2]) ** 2
                 ) ** (0.5)
        # donuts contains 1's and 0's organized in a donut shape
        # you apply 2 thresholds on circle to define the shape
        # slice jen s jednim kruhem
        slicecircle = circle < radius
        slicen = int(z_mm / voxelsize_mm[0])
        # slice s tim co už je v něm nastaveno
        slicetmp = data_seeds[slicen, :, :]
        # mport pdb; pdb.set_trace()

        slicetmp[slicecircle == 1] = label

        data_seeds[slicen, :, :] = slicetmp
    return data_seeds


def main():
    logger = logging.getLogger()

    logger.setLevel(logging.DEBUG)
    ch = logging.StreamHandler()
    logger.addHandler(ch)

    # create file handler which logs even debug messages
    # fh = logging.FileHandler('log.txt')
    # fh.setLevel(logging.DEBUG)
    # formatter = logging.Formatter(
    #     '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    # fh.setFormatter(formatter)
    # logger.addHandler(fh)
    # logger.debug('start')

    # input parser
    parser = argparse.ArgumentParser(
        description=__doc__
    )
    parser.add_argument(
        '-i', '--inputfile',
        default=None,
        required=True,
        help='input file'
    )
    parser.add_argument(
        '-d', '--debug', action='store_true',
        help='Debug mode')
    args = parser.parse_args()

    if args.debug:
        ch.setLevel(logging.DEBUG)



if __name__ == "__main__":
    main()
